#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import lxml.etree
import os
test_path = os.path.abspath(__file__)
test_dir_path = os.path.dirname(test_path) + os.sep

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.geos import GEOSGeometry
from django.contrib.messages.storage.fallback import FallbackStorage
from django.core.urlresolvers import reverse
from django.template import Context
from django.test import TestCase
from django.test.client import Client

from chimere.admin import managed_modified, MarkerAdmin
from chimere.models import Area, Icon, Importer, Category, SubCategory, Marker,\
                           Route, News
from chimere.forms import MarkerForm, AreaAdminForm
from chimere.templatetags.chimere_tags import display_news
from chimere.utils import ShapefileManager

def areas_setup():
    area_1 = Area.objects.create(name='area 1', urn='area-1', order=1,
                available=True,
                upper_left_corner='SRID=4326;POINT(-4.907753 48.507656)',
                lower_right_corner='SRID=4326;POINT(-4.049447 48.279688)')
    area_2 = Area.objects.create(name='area 2', urn='area-2', order=2,
                available=True,
                upper_left_corner='SRID=4326;POINT(-3 47.5)',
                lower_right_corner='SRID=4326;POINT(-2.5 47)')
    area_3 = Area.objects.create(name='area 3', urn='area-3', order=3,
                available=True,
                upper_left_corner='SRID=4326;POINT(-1.5 1.5)',
                lower_right_corner='SRID=4326;POINT(1.5 -1.5)')
    return [area_1, area_2, area_3]

def subcategory_setup():
    category = Category.objects.create(name='Main category',
                        available=True,
                        order=1,
                        description='')

    icon = Icon.objects.create(name='Default icon',
                               image='icons/marker.png',
                               height=25,
                               width=21)

    subcategory_1 = SubCategory.objects.create(category=category,
                        name='Subcategory 1',
                        available=True,
                        icon=icon,
                        order=1,
                        item_type='M',)

    subcategory_2 = SubCategory.objects.create(category=category,
                        name='Subcategory 2',
                        available=True,
                        icon=icon,
                        order=1,
                        item_type='M',)

    subcategory_3 = SubCategory.objects.create(category=category,
                        name='Subcategory 3',
                        available=True,
                        icon=icon,
                        order=1,
                        item_type='M',)

    subcategory_4 = SubCategory.objects.create(category=category,
                        name='Subcategory 4',
                        available=True,
                        icon=icon,
                        order=1,
                        item_type='M',)
    return [subcategory_1, subcategory_2, subcategory_3, subcategory_4]

def marker_setup(sub_categories=[]):
    if not sub_categories:
        sub_categories = subcategory_setup()
    current_date = datetime.datetime.now()
    markers = []
    marker_1 = Marker.objects.create(name="Marker 1", status='A',
                                     point='SRID=4326;POINT(-4.5 48.4)',
                                     available_date=current_date)
    marker_1.categories.add(sub_categories[0])
    markers.append(marker_1)
    marker_2 = Marker.objects.create(name="Marker 2", status='A',
                                     point='SRID=4326;POINT(-3.5 48.4)',
                                     available_date=current_date,
                                     start_date=current_date)
    marker_2.categories.add(sub_categories[1])
    markers.append(marker_2)
    marker_3 = Marker.objects.create(name="Marker 3", status='A',
                      point='SRID=4326;POINT(-4.5 48.45)',
                      available_date=current_date - datetime.timedelta(days=60),
                      start_date=current_date - datetime.timedelta(days=60),
                      end_date=current_date - datetime.timedelta(days=30))
    marker_3.categories.add(sub_categories[1])
    markers.append(marker_3)
    return markers

def route_setup(sub_categories=[]):
    if not sub_categories:
        sub_categories = subcategory_setup()
    current_date = datetime.datetime.now()
    routes = []
    route_1 = Route.objects.create(name="Route 1", status='A',
         has_associated_marker=True, route='SRID=4326;LINESTRING(-1 1, 1 -1)')
    route_1.categories.add(sub_categories[0])
    routes.append(route_1)
    route_2 = Route.objects.create(name="Route 2", status='A',
        has_associated_marker=False, route='SRID=4326;LINESTRING(0 0, 2 2)')
    route_2.categories.add(sub_categories[1])
    routes.append(route_2)
    return routes

class ImporterTest:
    def test_get(self):
        nb_by_cat = {}
        for importer, awaited_nb in self.marker_importers:
            nb, nb_updated, res = importer.manager.get()
            if awaited_nb == None:
                continue
            self.assertEqual(nb, awaited_nb)
            self.assertEqual(nb_updated, 0)
            for cat in importer.categories.all():
                if cat not in nb_by_cat:
                    nb_by_cat[cat] = 0
                nb_by_cat[cat] += nb
        for cat in nb_by_cat:
            nb = max([Marker.objects.filter(categories__pk=cat.pk).count(),
                      Route.objects.filter(categories__pk=cat.pk).count()])
            self.assertEqual(nb_by_cat[cat], nb)
        # update
        for importer, awaited_nb in self.marker_importers:
            importer.overwrite = True
            importer.save()
            nb, nb_updated, res = importer.manager.get()
            if awaited_nb == None:
                continue
            self.assertEqual(nb, 0)
        # manage overwrite
        for importer, awaited_nb in self.marker_importers:
            if not awaited_nb:
                continue
            # mimic the modification of one item
            for cls in (Marker, Route):
                items = cls.objects.filter(
                                    categories=importer.categories.all()[0]
                                    ).order_by('-pk').all()
                if items.count():
                    item = items.all()[0]
                    item.import_version = 99999 # fake version number
                    item.save()
                    # as when the import_version it is considered as an import
                    # modification force the modification flag
                    item.modified_since_import = True
                    item.save()
            importer.overwrite = False
            importer.save()
            nb, nb_updated, res = importer.manager.get()
            if awaited_nb == None:
                continue
            self.assertEqual(nb, 1)

class KMLImporterTest(TestCase, ImporterTest):
    def setUp(self):
        subcategories = subcategory_setup()
        importer1 = Importer.objects.create(importer_type='KML',
            source=test_dir_path+'tests/sample.kml',
            filtr="Category 1")
        importer1.categories.add(subcategories[0])

        importer2 = Importer.objects.create(importer_type='KML',
         source=test_dir_path+'tests/sample.kml',
         filtr="Subcategory 1", associate_marker_to_way=True)
        importer2.categories.add(subcategories[1])

        importer3 = Importer.objects.create(importer_type='KML',
         source=test_path+'tests/sample.kml',
         filtr="Subcategory 3")
        importer3.categories.add(subcategories[2])

        importer4 = Importer.objects.create(importer_type='KML',
         source=test_dir_path+'tests/sample.kml.zip', zipped=True,
         default_description="Default description")
        importer4.categories.add(subcategories[3])

        self.marker_importers = [(importer1, 1), (importer2, 3), (importer3, 0),
                                 (importer4, 4)]

    def test_defaultdescription(self):
        Marker.objects.all().delete()
        importer = self.marker_importers[-1][0]
        importer.overwrite = True
        importer.save()
        importer.manager.get()
        last_marker = Marker.objects.order_by('-pk').all()[0]
        self.assertEqual(last_marker.description,
                         importer.default_description)
        # don't overwrite description on update
        new_desc = u"Description set by an user"
        last_marker.description = new_desc
        last_marker.save()
        importer.manager.get()
        last_marker = Marker.objects.order_by('-pk').all()[0]
        self.assertEqual(last_marker.description,
                         new_desc)

class ShapefileImporterTest(TestCase, ImporterTest):
    def setUp(self):
        self.subcategories = subcategory_setup()
        importer = Importer.objects.create(importer_type='SHP',
            source=test_dir_path+'tests/sample_nodes.shp.zip', zipped=True)
        importer.categories.add(self.subcategories[0])
        importer2 = Importer.objects.create(importer_type='SHP',
            source=test_dir_path+'tests/sample_ways.shp.zip',
            zipped=True)
        importer2.categories.add(self.subcategories[1])

        self.marker_importers = [(importer, 29),
                                 (importer2, 5),]
        self.markers = marker_setup()

    def test_export(self):
        filename, zip_stream = ShapefileManager.export(Marker.objects.all())

    def test_associate_marker_to_way(self):
        importer, nb = self.marker_importers[1]

        importer.associate_marker_to_way = True
        importer.save()
        nb, nb_updated, res = importer.manager.get()
        nb = Marker.objects.filter(categories__pk=self.subcategories[1].pk
                                   ).count()
        self.assertEqual(nb, 5)
        Marker.objects.filter(categories__pk=self.subcategories[1].pk).delete()
        Route.objects.filter(categories__pk=self.subcategories[1].pk).delete()

        importer.associate_marker_to_way = False
        importer.save()
        nb, nb_updated, res = importer.manager.get()
        nb = Marker.objects.filter(categories__pk=self.subcategories[1].pk
                                   ).count()
        self.assertEqual(nb, 0)

class OSMImporterTest(TestCase, ImporterTest):
    def setUp(self):
        subcategories = subcategory_setup()
        markers = marker_setup()
        importer1 = Importer.objects.create(importer_type='OSM',
            source=test_dir_path+'tests/sample_nodes.osm')
        importer1.categories.add(subcategories[0])

        importer2 = Importer.objects.create(importer_type='OSM',
            source=test_dir_path+'tests/sample_ways.osm')
        importer2.categories.add(subcategories[1])

        self.marker_importers = [(importer1, 19), (importer2, 8)]

class GeoRSSImporterTest(TestCase, ImporterTest):
    def setUp(self):
        subcategories = subcategory_setup()
        importer1 = Importer.objects.create(importer_type='RSS',
                         source=test_dir_path+'tests/georss_simple.xml')
        importer1.categories.add(subcategories[0])
        importer2 = Importer.objects.create(importer_type='RSS',
                         source=test_dir_path+'tests/eqs7day-M5.xml')
        importer2.categories.add(subcategories[1])

        self.marker_importers = [(importer1, 1), (importer2, 32)]

class FeedsTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()
        self.markers = marker_setup()

    def test_rss(self):
        # global
        url = reverse('chimere:feeds-global')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        doc = lxml.etree.fromstring(response.content)
        self.assertEqual(int(doc.xpath('count(//item)')), len(self.markers))
        url = reverse('chimere:feeds-areaid', args=('', self.areas[0].pk))
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        doc = lxml.etree.fromstring(response.content)
        self.assertEqual(int(doc.xpath('count(//item)')), 2)

class AdminTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()
        self.markers = marker_setup()
        self.adminuser = User.objects.create_superuser('admin',
                                                  'admin@test.com',
                                                  'pass')
        self.client.login(username='admin', password='pass')

    def test_actions(self):
        q_markers = Marker.objects.filter(pk__in=[m.pk for m in self.markers])
        # disable
        response = self.client.post('/admin/chimere/marker/',
                       data={'action':['disable'],
                             '_selected_action':[unicode(m.pk)
                                                 for m in self.markers],
                            })
        self.assertEqual(q_markers.count(),
                         q_markers.filter(status='D').count())
        # validate
        response = self.client.post('/admin/chimere/marker/',
                       data={'action':['validate'],
                             '_selected_action':[unicode(m.pk)
                                                 for m in self.markers]
                            })
        self.assertEqual(q_markers.count(),
                         q_markers.filter(status='A').count())


class MarkerFormTest(TestCase):
    def setUp(self):
        self.subcategories = subcategory_setup()

    def test_marker_creation(self):
        current_date = datetime.datetime.now()
        # end_date before start_date
        data = {'name':"Marker 1", 'status':'A', 'available_date':current_date,
                'point':'SRID=4326;POINT(-4.5 48.4)', 'start_date':current_date,
                'end_date':current_date - datetime.timedelta(1),
                'categories':[self.subcategories[0].pk]}
        form = MarkerForm(data)
        self.assertEqual(form.is_valid(), False)

class AreaTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()

    def test_area_availability(self):
        area_1 = self.areas[0]
        area_1.available = False
        area_1.save()
        response = self.client.get('/%s/' % area_1.urn)
        self.assertRedirects(response, '/')

class AreaAdminFormTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()

    def test_area_default(self):
        area_1, area_2 = self.areas[0], self.areas[1]
        area_1.default = True
        area_1.save()
        area_2.default = True
        area_2.save()
        area_1 = Area.objects.get(urn=area_1.urn)
        self.assertEqual(area_1.default, False)

    def test_area_creation(self):
        base_data = {'name':u'New test', 'order':3, 'available':True,
                'urn':'area-new',
                'upper_left_lat':48.5,
                'upper_left_lon':-5,
                'lower_right_lat':48,
                'lower_right_lon':-4,
                'upper_left_corner':'SRID=4326;POINT(0 0)',
                'lower_right_corner':'SRID=4326;POINT(0 0)'}
        # order already given
        data = base_data.copy()
        data['order'] = self.areas[0].order
        form = AreaAdminForm(data)
        self.assertEqual(form.is_valid(), False)
        # update an already existing area
        data = base_data.copy()
        data['order'] = self.areas[0].order
        form = AreaAdminForm(data, instance=self.areas[0])
        self.assertEqual(form.is_valid(), True)
        # empty area
        data = base_data.copy()
        data.update({'upper_left_lat': 0,
                     'upper_left_lon': 0,
                     'lower_right_lat': 0,
                     'lower_right_lon': 0})
        form = AreaAdminForm(data)
        self.assertEqual(form.is_valid(), False)

class DynamicCategoryTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()
        subcategories = subcategory_setup()
        self.markers = marker_setup(subcategories)
        self.routes = route_setup(subcategories)

    def test_dynamic_evaluation(self):
        cats = self.areas[0].getCategories(status='A', filter_available=True)
        self.assertEqual(len(cats), 1)
        cats = self.areas[2].getCategories(status='A', filter_available=True)
        self.assertEqual(len(cats), 2)

class NewsTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()
        self.markers = marker_setup()
        current_date = datetime.datetime.now()
        marker = Marker.objects.create(name="Marker 4", status='A',
                      point='SRID=4326;POINT(-4.5 48.45)',
                      available_date=current_date - datetime.timedelta(days=90),
                      start_date=current_date - datetime.timedelta(days=90),
                      end_date=None)
        self.news = []
        self.news.append(News.objects.create(title=u"Test news 1",
                                             available=True))
        self.news.append(News.objects.create(title=u"Test news 2",
                                             available=False))

    def test_news_display(self):
        context = display_news(Context({}))
        self.assertEqual(len(context['news_lst']), 2)
        context = display_news(Context({'area_name':'area-2'}))
        self.assertEqual(len(context['news_lst']), 1)

class RapprochementTest(TestCase):
    def setUp(self):
        self.areas = areas_setup()
        self.subcategories = subcategory_setup()
        self.markers = marker_setup(self.subcategories)
        self.routes = route_setup(self.subcategories)
        self.adminuser = User.objects.create_superuser('admin',
                                                  'admin@test.com',
                                                  'pass')
        self.client.login(username='admin', password='pass')

    def test_managed_modified_markers(self):
        ref_marker = self.markers[0]
        new_vals = {'name':"Marker 1 - modified",
                    'point':GEOSGeometry('SRID=4326;POINT(-4 48)')}
        values = {'status':'M', 'ref_item':ref_marker}
        values.update(new_vals)
        modified_marker = Marker.objects.create(**values)
        modified_marker.categories.add(ref_marker.categories.all()[0])
        response = self.client.post('/admin/chimere/marker/',
                       data={'action':['managed_modified'],
                             'index':0, 'rapprochement':1,
                             'name':1, 'point':1,
                             '_selected_action':[unicode(ref_marker.pk)]
                            })
        ref_marker = Marker.objects.get(pk=ref_marker.pk)
        self.assertEqual(Marker.objects.filter(ref_item=ref_marker,
                                               status='M').count(), 0)
        for k in new_vals:
            self.assertEqual(getattr(ref_marker, k), new_vals[k])

    def test_managed_modified_imported_markers(self):
        ref_marker = self.markers[0]
        new_vals = {'name':"Marker 1 - modified",
                    'point':GEOSGeometry('SRID=4326;POINT(-4 48)')}
        values = {'status':'I', 'ref_item':ref_marker, 'import_version':42}
        values.update(new_vals)
        modified_marker = Marker.objects.create(**values)
        self.assertNotEqual(ref_marker.import_version,
                            modified_marker.import_version)
        modified_marker.categories.add(ref_marker.categories.all()[0])
        response = self.client.post('/admin/chimere/marker/',
                       data={'action':['managed_modified'],
                             'index':0, 'rapprochement':1,
                             'name':1, 'point':1,
                             '_selected_action':[unicode(ref_marker.pk)]
                            })
        ref_marker = Marker.objects.get(pk=ref_marker.pk)
        self.assertEqual(Marker.objects.filter(ref_item=ref_marker,
                                               status='I').count(), 0)
        for k in new_vals.keys() + ['import_version']:
            self.assertEqual(getattr(ref_marker, k), values[k])

    def test_managed_modified_routes(self):
        ref_route = self.routes[0]
        new_vals = {'name':"Route 1 - modified",
                    'route':GEOSGeometry('SRID=4326;LINESTRING(1 1,2 2)')}
        values = {'status':'M', 'ref_item':ref_route,
                  'has_associated_marker':True}
        values.update(new_vals)
        modified_route = Route.objects.create(**values)
        modified_route.categories.add(self.subcategories[1])
        response = self.client.post('/admin/chimere/route/',
                       data={'action':['managed_modified'],
                             'index':0, 'rapprochement':1,
                             'name':1, 'route':1, 'categories':1,
                             '_selected_action':[unicode(ref_route.pk)]
                            })
        ref_route = Route.objects.get(pk=ref_route.pk)
        self.assertEqual(Route.objects.filter(ref_item=ref_route,
                                               status='M').count(), 0)
        self.assertEqual(ref_route.name, new_vals['name'])
        self.assertEqual(ref_route.route.wkt, new_vals['route'].wkt)
        self.assertEqual(ref_route.categories.all()[0], self.subcategories[1])
        self.assertEqual(ref_route.associated_marker.all()[0].name,
                         ref_route.name)

class RouteTest(TestCase):
    def setUp(self):
        self.subcategories = subcategory_setup()

    def test_route_creation(self):
        route_1 = Route.objects.create(name='Route 1',
                      route='SRID=4326;LINESTRING (30 10, 10 30, 40 40)')
        self.assertEqual(Marker.objects.filter(route=route_1).count(), 1)
        route_2 = Route.objects.create(name='Route 1',
                      route='SRID=4326;LINESTRING (30 10, 10 30, 40 40)',
                      has_associated_marker=False)
        self.assertEqual(Marker.objects.filter(route=route_2).count(), 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Models description
"""
import os, datetime, pyexiv2, re, string
import simplejson as json
from lxml import etree
from PIL import Image
from subprocess import Popen, PIPE
from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib.auth.models import User, Permission, ContentType, Group
from django.contrib.gis.db import models
from django.contrib.gis.gdal import SpatialReference
from django.core.files import File
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.db.models.signals import post_save, pre_save, m2m_changed
from django.template import defaultfilters
from django.utils.translation import ugettext_lazy as _

from chimere.widgets import PointField, RouteField, SelectMultipleField, \
                            TextareaWidget, DatePickerWidget
from chimere.managers import BaseGeoManager
from chimere.utils import KMLManager, OSMManager, ShapefileManager, \
                          GeoRSSManager, CSVManager

class News(models.Model):
    """News of the site
    """
    title = models.CharField(_(u"Name"), max_length=150)
    available = models.BooleanField(_(u"Available"))
    date = models.DateField(_(u"Date"), auto_now_add=True)
    content = models.TextField()
    areas = SelectMultipleField('Area', verbose_name=_(u"Associated areas"),
                                blank=True, null=True)
    def __unicode__(self):
        ordering = ["-date"]
        return self.title
    class Meta:
        verbose_name = _(u"News")
        verbose_name_plural = _(u"News")

class TinyUrl(models.Model):
    """Tinyfied version of permalink parameters
    """
    parameters = models.CharField(_(u"Parameters"), max_length=500)
    def __unicode__(self):
        return self.parameters
    class Meta:
        verbose_name = _(u"TinyUrl")
    digits = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
    base = len(digits)

    @classmethod
    def getParametersByUrn(cls, urn):
        c_id = 0
        for idx, char in enumerate(reversed(urn)):
            c_id += cls.digits.index(char)*pow(cls.base, idx)
        try:
            params = cls.objects.get(id=c_id).parameters
        except cls.DoesNotExist:
            return ''
        return params

    @classmethod
    def getUrnByParameters(cls, parameters):
        try:
            obj = cls.objects.get(parameters=parameters)
        except cls.DoesNotExist:
            obj = cls(parameters=parameters)
            obj.save()
        n = obj.id
        urn = ''
        while 1:
            idx = n % cls.base
            urn = cls.digits[idx] + urn
            n = n / cls.base
            if n == 0:
                break
        return urn

class ColorTheme(models.Model):
    """Color theme
    """
    name = models.CharField(_(u"Name"), max_length=150)
    def __unicode__(self):
        return self.name
    class Meta:
        verbose_name = _(u"Color theme")

class Color(models.Model):
    """Color
    """
    code = models.CharField(_(u"Code"), max_length=6)
    order = models.IntegerField(_(u"Order"))
    color_theme = models.ForeignKey(ColorTheme, verbose_name=_(u"Color theme"))
    def __unicode__(self):
        return self.code
    class Meta:
        ordering = ["order"]
        verbose_name = _(u"Color")

class Category(models.Model):
    """Category of Point Of Interest (POI)
    """
    name = models.CharField(_(u"Name"), max_length=150)
    available = models.BooleanField(_(u"Available"))
    order = models.IntegerField(_(u"Order"))
    description = models.TextField(blank=True, null=True)
    def __unicode__(self):
        return self.name
    class Meta:
        ordering = ["order"]
        verbose_name = _(u"Category")

class Icon(models.Model):
    '''Icon
    '''
    name = models.CharField(_(u"Name"), max_length=150)
    image = models.ImageField(_(u"Image"), upload_to='icons',
                              height_field='height', width_field='width')
    height = models.IntegerField(_(u"Height"))
    width = models.IntegerField(_(u"Width"))
    def __unicode__(self):
        return self.name
    class Meta:
        verbose_name = _(u"Icon")

class SubCategory(models.Model):
    '''Sub-category
    '''
    category = models.ForeignKey(Category, verbose_name=_(u"Category"),
                                 related_name='subcategories')
    name = models.CharField(_(u"Name"), max_length=150)
    available = models.BooleanField(_(u"Available"), default=True)
    submission = models.BooleanField(_(u"Available for submission"),
                                     default=True)
    icon = models.ForeignKey(Icon, verbose_name=_(u"Icon"))
    color_theme = models.ForeignKey(ColorTheme, verbose_name=_(u"Color theme"),
                                    blank=True, null=True)
    order = models.IntegerField(_(u"Order"), default=1000)
    TYPE = (('M', _(u'Marker')),
            ('R', _(u'Route')),
            ('B', _(u'Both')),)
    item_type = models.CharField(_(u"Item type"), max_length=1, choices=TYPE)
    def __unicode__(self):
        return u"%s / %s" % (self.category.name, self.name)
    class Meta:
        ordering = ["category", "order"]
        verbose_name = _(u"Sub-category")
        verbose_name_plural = _(u"Sub-categories")

    @classmethod
    def getAvailable(cls, item_types=None, area_name=None, public=False):
        '''Get list of tuples with first the category and second the associated
        subcategories
        '''
        sub_categories = {}
        subcategories = cls.objects.filter(category__available=True)
        if not item_types:
            subcategories = subcategories.filter(available=True)
        else:
            subcategories = subcategories.filter(item_type__in=item_types)
        if public:
            subcategories = subcategories.filter(submission=True)
        selected_cats = []
        if area_name:
            area = Area.objects.get(urn=area_name)
            # if there some restrictions with categories limit them
            if area.subcategories.count():
                sub_ids = [sub.id for sub in area.subcategories.all()]
                subcategories = subcategories.filter(id__in=sub_ids)
            selected_cats = [subcat.pk
                             for subcat in area.default_subcategories.all()]
        for sub_category in subcategories.order_by('order'):
            if sub_category.category not in sub_categories:
                sub_categories[sub_category.category] = []
            if sub_category.id in selected_cats:
                sub_category.selected = True
                sub_category.category.selected = True
            sub_categories[sub_category.category].append(sub_category)

        subcategories = [(cat, subcats) \
                           for cat, subcats in sub_categories.items()]
        get_cat_order = lambda cat_tuple: cat_tuple[0].order
        subcategories = sorted(subcategories, key=get_cat_order)
        return subcategories

IMPORTERS = {'KML':KMLManager,
             'OSM':OSMManager,
             'SHP':ShapefileManager,
             'RSS':GeoRSSManager,
             'CSV':CSVManager
             }

IMPORTER_CHOICES = (('KML', 'KML'),
                    ('OSM', 'OSM'),
                    ('SHP', 'Shapefile'),
                    ('RSS', 'GeoRSS'),
                    ('CSV', 'CSV')
                    )

IMPORTER_CHOICES_DICT = dict(IMPORTER_CHOICES)

class Importer(models.Model):
    '''
    Data importer for a specific subcategory
    '''
    importer_type = models.CharField(_(u"Importer type"), max_length=4,
                                     choices=IMPORTER_CHOICES)
    filtr = models.CharField(_(u"Filter"), max_length=200,
                             blank=True, null=True)
    source = models.CharField(_(u"Web address"), max_length=200,
                              blank=True, null=True)
    source_file = models.FileField(_(u"Source file"),
                        upload_to='import_files', blank=True, null=True)
    default_name = models.CharField(_(u"Name by default"), max_length=200,
                                    blank=True, null=True)
    srid = models.IntegerField(_(u"SRID"), blank=True, null=True)
    zipped = models.BooleanField(_(u"Zipped file"), default=False)
    overwrite = models.BooleanField(_(u"Overwrite existing data"),
                                    default=False)
    get_description = models.BooleanField(_(u"Get description from source"),
                                          default=False)
    default_description = models.TextField(_(u"Default description"),
                                           blank=True, null=True)
    origin = models.CharField(_(u"Origin"), max_length=100,
                              blank=True, null=True)
    license = models.CharField(_(u"License"), max_length=100,
                               blank=True, null=True)
    categories = SelectMultipleField(SubCategory,
                      verbose_name=_(u"Associated subcategories"))
    state = models.CharField(_(u"State"), max_length=200,
                             blank=True, null=True)
    associate_marker_to_way = models.BooleanField(_(u"Automatically associate "\
                                           u"a marker to a way"), default=False)

    class Meta:
        verbose_name = _(u"Importer")

    def __unicode__(self):
        vals = [IMPORTER_CHOICES_DICT[self.importer_type],
                self.source, self.source_file.name,
                u", ".join([unicode(cat) for cat in self.categories.all()]),
                self.default_name]
        return u' %d: %s' % (self.pk, u" - ".join([unicode(v)
                                                   for v in vals if v]))

    @property
    def manager(self):
        return IMPORTERS[self.importer_type](self)

    def display_categories(self):
        return u"\n".join([cat.name for cat in self.categories.all()])

class GeographicItem(models.Model):
    name = models.CharField(_(u"Name"), max_length=150)
    categories = SelectMultipleField(SubCategory)
    submiter_session_key = models.CharField(_(u"Submitter session key"),
                                        blank=True, null=True, max_length=40)
    submiter_name = models.CharField(_(u"Submitter name or nickname"),
                                     blank=True, null=True, max_length=40)
    submiter_email = models.EmailField(_(u"Submitter email"), blank=True,
                                       null=True)
    submiter_comment = models.TextField(_(u"Submitter comment"), max_length=200,
                                        blank=True, null=True)
    STATUS = (('S', _(u'Submited')),
              ('A', _(u'Available')),
              ('M', _(u'Modified')),
              ('D', _(u'Disabled')),
              ('I', _(u'Imported')))
    STATUS_DCT = dict(STATUS)
    status = models.CharField(_(u"Status"), max_length=1, choices=STATUS)
    import_key = models.CharField(_(u"Import key"), max_length=200,
                                  blank=True, null=True)
    import_version = models.IntegerField(_(u"Import version"),
                                         blank=True, null=True)
    import_source = models.CharField(_(u"Source"), max_length=200,
                                     blank=True, null=True)
    modified_since_import = models.BooleanField(_(u"Modified since last import"),
                                                default=True)
    not_for_osm = models.BooleanField(_(u"Not to be exported to OSM"),
                                      default=False)
    origin = models.CharField(_(u"Origin"), max_length=100,
                              blank=True, null=True)
    license = models.CharField(_(u"License"), max_length=100,
                               blank=True, null=True)
    start_date = models.DateField(_(u"Start date"), blank=True, null=True,
        help_text=_(u"Not mandatory. Set it for dated item such as event. "\
                    u"Format YYYY-MM-DD"))
    end_date = models.DateField(_(u"End date"), blank=True, null=True,
        help_text=_(u"Not mandatory. Set it only if you have a multi-day "\
                    u"event. Format YYYY-MM-DD"))
    class Meta:
        abstract = True

    def get_key(self, key):
        key_vals = self.import_key.split(';')
        for k_v in key_vals:
            if k_v.startswith(key+':'):
                return k_v.split(':')[1]

    def set_key(self, key, value):
        new_keys, _set = '', None
        key_vals = self.import_key.split(';') if self.import_key else []
        for k_v in key_vals:
            if ':' not in k_v:
                continue
            k, v = k_v.split(':')
            if k == key:
                _set = True
                new_keys += '%s:%s;' % (k, value)
            else:
                new_keys += '%s:%s;' % (k, v)
        if not _set:
            new_keys += '%s:%s;' % (key, value)
        self.import_key = new_keys
        self.save()

    def has_modified(self):
        if (self.ref_item and self.ref_item != self) \
           or self.__class__.objects.filter(ref_item=self
                                ).exclude(pk=self.pk).count():
            return True
        return False

    @classmethod
    def properties(cls):
        return [pm for pm in PropertyModel.objects.filter(available=True)]

def property_setter(cls, propertymodel):
    def setter(self, value):
        marker = self
        if cls == Route:
            if not self.associated_marker.objects.count():
                return
            marker = self.associated_marker.objects.all()[0]
        marker.setProperty(propertymodel, value)
    return setter

class Marker(GeographicItem):
    '''Marker for a POI
    '''
    ref_item = models.ForeignKey("Marker", blank=True, null=True,
            verbose_name=_(u"Reference marker"), related_name='submited_marker')
    point = PointField(_(u"Localisation"),
                       srid=settings.CHIMERE_EPSG_DISPLAY_PROJECTION)
    available_date = models.DateTimeField(_(u"Available Date"), blank=True,
                                          null=True) # used by feeds
    route = models.ForeignKey(u"Route", blank=True, null=True,
                              related_name='associated_marker')
    description = models.TextField(_(u"Description"), blank=True, null=True)
    objects = models.GeoManager()

    def __unicode__(self):
        return self.name

    def __init__(self, *args, **kwargs):
        super(Marker, self).__init__(*args, **kwargs)
        # add read attributes for properties
        for pm in self.properties():
            attr_name = pm.getAttrName()
            if not hasattr(self, attr_name):
                val = ''
                property = self.getProperty(pm)
                if property:
                    val = property.python_value
                setattr(self, attr_name, val)
            if not hasattr(self, attr_name + '_set'):
                setattr(self, attr_name + '_set',
                              property_setter(self.__class__, pm))

    def get_init_multi(self):
        multis = [forms.model_to_dict(multi)
                  for multi in self.multimedia_files.all()]
        return multis

    def get_init_picture(self):
        picts = [forms.model_to_dict(pict)
                 for pict in self.pictures.all()]
        return picts

    @property
    def multimedia_items(self):
        pict = list(self.pictures.filter(miniature=False).all())
        mm = list(self.multimedia_files.filter(miniature=False).all())
        items = [(item.order, item) for item in pict + mm]
        return [item for order, item in sorted(items)]

    @property
    def default_pictures(self):
        return list(self.pictures.filter(miniature=True).order_by('order'))

    @property
    def default_multimedia_items(self):
        return list(self.multimedia_files.filter(miniature=True
                                                 ).order_by('order'))

    @property
    def date(self):
        if settings.CHIMERE_DAYS_BEFORE_EVENT:
            return self.start_date

    @property
    def geometry(self):
        return self.point.wkt

    @property
    def geom_attr(self):
        return 'point'

    class Meta:
        ordering = ('status', 'name')
        verbose_name = _(u"Point of interest")

    def getLatitude(self):
        '''Return the latitude
        '''
        return self.point.y

    def getLongitude(self):
        '''Return the longitude
        '''
        return self.point.x

    def getProperty(self, propertymodel, safe=None):
        """Get the property of an associated property model.
        If safe set to True, verify if the property is available
        """
        if safe and not propertymodel.available:
            return
        try:
            property = Property.objects.get(propertymodel=propertymodel,
                                            marker=self)
        except Property.DoesNotExist:
            return
        return property

    def getProperties(self):
        """Get all the property availables
        """
        properties = []
        for pm in PropertyModel.objects.filter(available=True):
            property = self.getProperty(pm)
            if property:
                properties.append(property)
        return properties

    def setProperty(self, pm, value):
        u"""
        Set a property
        """
        properties = Property.objects.filter(marker=self,
                                             propertymodel=pm).all()
        # in case of multiple edition as the same time delete arbitrary
        # the others
        if len(properties) > 1:
            for property in properties[1:]:
                property.delete()
        # new property
        if not properties:
            new_property = Property.objects.create(marker=self,
                                propertymodel=pm,
                                value=value)
            new_property.save()
        else:
            property = properties[0]
            property.value = value
            property.save()

    def saveProperties(self, values):
        """
        Save properties
        """
        for propertymodel in PropertyModel.objects.filter(available=True):
            val = u""
            if unicode(propertymodel.id) in values:
                val = values[unicode(propertymodel.id)]
            self.setProperty(propertymodel, val)

    def getGeoJSON(self, categories_id=[]):
        '''Return a GeoJSON string
        '''
        jsons = []
        for cat in self.categories.all():
            if categories_id and cat.id not in categories_id:
                continue
            items = {'id':self.id, 'name':json.dumps(self.name),
                     'geometry':self.point.geojson,
                     'icon_path':cat.icon.image,
                     'icon_width':cat.icon.image.width,
                     'icon_height':cat.icon.image.height,}
            jsons.append(u'{"type":"Feature", "geometry":%(geometry)s, '\
                u'"properties":{"pk": %(id)d, "name": %(name)s, '\
                u'"icon_path":"%(icon_path)s", "icon_width":%(icon_width)d, '\
                u'"icon_height":%(icon_height)d}}' % items)
        return ",".join(jsons)

    @property
    def default_category(self):
        # Should we select only available ones ?
        # Should we catch if not exists ?
        return self.categories.all()[0]

    def get_absolute_url(self, area_name=''):
        parameters = 'current_feature=%d&checked_categories=%s' % (self.id,
                                                    self.default_category.pk)
        urn = TinyUrl.getUrnByParameters(parameters)
        area_name = area_name + '/' if area_name else ''
        url = reverse('chimere:tiny', args=[area_name, urn])
        return url


PRE_ATTRS = {
    'Marker':('name', 'geometry', 'import_version', 'modified_since_import'),
    'Route':('name', 'geometry', 'import_version', 'modified_since_import'),
    'Area':('urn', 'name'),
    }
def geometry_pre_save(cls, pre_save_geom_values):
    def geom_pre_save(sender, **kwargs):
        if not kwargs['instance'] or not kwargs['instance'].pk:
            return
        instance = kwargs['instance']
        try:
            instance = cls.objects.get(pk=instance.pk)
            pre_save_geom_values[instance.pk] = [getattr(instance, attr)
                                            for attr in PRE_ATTRS[cls.__name__]]
        except ObjectDoesNotExist:
            pass
    return geom_pre_save

pre_save_marker_values = {}
def marker_pre_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    geometry_pre_save(Marker, pre_save_marker_values)(sender, **kwargs)
pre_save.connect(marker_pre_save, sender=Marker)

def geometry_post_save(pre_save_geom_values):
    def geom_post_save(sender, **kwargs):
        if not kwargs['instance'] \
           or kwargs['instance'].pk not in pre_save_geom_values:
            return
        instance = kwargs['instance']
        name, geometry, import_version, modified_since_import = \
                                    pre_save_geom_values[instance.pk]
        # force the reinit of modified_since_import
        if modified_since_import != instance.modified_since_import:
            return
        if (instance.import_version != import_version
           and instance.modified_since_import):
            instance.modified_since_import = False
            instance.save()
            return
        if instance.modified_since_import:
            return
        if instance.name != name or instance.geometry != geometry:
            instance.modified_since_import = True
            instance.save()
    return geom_post_save
def marker_post_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    geometry_post_save(pre_save_marker_values)(sender, **kwargs)
post_save.connect(marker_post_save, sender=Marker)

class MultimediaType(models.Model):
    MEDIA_TYPES = (('A', _(u"Audio")),
                   ('V', _(u"Video")),
                   ('I', _(u"Image")),
                   ('O', _(u"Other")),)
    media_type = models.CharField(_(u"Media type"), max_length=1,
                                  choices=MEDIA_TYPES)
    name = models.CharField(_(u"Name"), max_length=150)
    mime_type = models.CharField(_(u"Mime type"), max_length=50, blank=True,
                                 null=True)
    iframe = models.BooleanField(_(u"Inside an iframe"), default=False)
    available = models.BooleanField(_(u"Available"), default=True)

    class Meta:
        verbose_name = _(u"Multimedia type")
        verbose_name_plural = _(u"Multimedia types")

    def __unicode__(self):
        return self.name

    @classmethod
    def get_tuples(cls):
        vals = cls.objects.filter(available=True).order_by('media_type',
                                                           'name')
        tuples, c_tpe = [('', _("Automatic recognition"))], None
        media_type_dct = dict(cls.MEDIA_TYPES)
        for tpe, pk, name in [(media_type_dct[v.media_type], v.pk, v.name)
                              for v in vals]:
            if not c_tpe or c_tpe != tpe:
                c_tpe = tpe
                tuples.append([tpe, []])
            tuples[-1][1].append((pk, name))
        return tuples

IFRAME_LINKS = {
    'youtube':((re.compile(r'youtube.com\/watch\?v=([A-Za-z0-9_-]*)'),
                re.compile(r'youtu.be\/([A-Za-z0-9_-]*)'),
                re.compile(r'youtube.com\/embed\/([A-Za-z0-9_-]*)')),
                 "http://www.youtube.com/embed/%s"),
    'dailymotion':(
        (re.compile(r'dailymotion.com/video/([A-Za-z0-9]*)_[A-Za-z0-9_-]*'),
         re.compile(r'dailymotion.com/embed/video/([A-Za-z0-9]*)'),
         re.compile("http://www.dailymotion.com/embed/video/%s")),
         'http://www.dailymotion.com/embed/video/%s'),
    'vimeo':((re.compile(r'vimeo.com/video/([A-Za-z0-9]*)'),
              re.compile(r'vimeo.com/([A-Za-z0-9]*)'),),
              "http://player.vimeo.com/video/%s")
}

class MultimediaExtension(models.Model):
    name = models.CharField(_(u"Extension name"), max_length=6)
    multimedia_type = models.ForeignKey(MultimediaType,
                        verbose_name=_(u"Associated multimedia type"),
                        related_name='extensions')

    class Meta:
        verbose_name = _(u"Multimedia extension")
        verbose_name_plural = _(u"Multimedia extensions")

    def __unicode__(self):
        return self.name

class MultimediaFile(models.Model):
    name = models.CharField(_(u"Name"), max_length=150)
    url = models.URLField(_(u"Url"), max_length=200)
    order = models.IntegerField(_(u"Order"), default=1)
    multimedia_type = models.ForeignKey(MultimediaType, blank=True, null=True)
    miniature = models.BooleanField(_(u"Display inside the description?"),
                                  default=settings.CHIMERE_MINIATURE_BY_DEFAULT)
    marker = models.ForeignKey(Marker, related_name='multimedia_files')

    class Meta:
        verbose_name = _(u"Multimedia file")
        verbose_name_plural = _(u"Multimedia files")

    def __unicode__(self):
        return self.name or u""

def multimediafile_post_save(sender, **kwargs):
    if not kwargs['instance'] or not kwargs['created']:
        return
    multimediafile = kwargs['instance']
    # auto recognition of file types
    if not multimediafile.multimedia_type:
        url = multimediafile.url
        for mm_type in IFRAME_LINKS:
            res, embeded_url = IFRAME_LINKS[mm_type]
            if [r for r in res if r.search(url)]:
                multimedia_type = MultimediaType.objects.get(
                                    name__iexact=mm_type)
                multimediafile.multimedia_type = multimedia_type
        if not multimediafile.multimedia_type:
            ext = url.split(".")[-1]
            q = MultimediaExtension.objects.filter(name__iendswith=ext)
            if q.count():
                multimediafile.multimedia_type = q.all()[0].multimedia_type
            else:
                # default to an iframe
                multimediafile.multimedia_type = \
                   MultimediaType.objects.filter(name__iexact='iframe').all()[0]
    # manage iframe of video providers
    if multimediafile.multimedia_type.name.lower() in IFRAME_LINKS:
        regexps, lnk = IFRAME_LINKS[multimediafile.multimedia_type.name.lower()]
        key = None
        for regexp in regexps:
            key = regexp.findall(multimediafile.url)
            if key:
                key = key[0]
                break
        if key:
            multimediafile.url = lnk % key

    mfs = MultimediaFile.objects.filter(marker=multimediafile.marker).exclude(
                                        pk=multimediafile.pk).order_by('order')
    for idx, mf in enumerate(mfs.all()):
        mf.order = idx + 1
        mf.save()
    multimediafile.order = mfs.count() + 1
    multimediafile.save()
post_save.connect(multimediafile_post_save, sender=MultimediaFile)

class PictureFile(models.Model):
    name = models.CharField(_(u"Name"), max_length=150)
    picture = models.ImageField(_(u"Image"), upload_to='pictures',
                                height_field='height', width_field='width')
    height = models.IntegerField(_(u"Height"), blank=True, null=True)
    width = models.IntegerField(_(u"Width"), blank=True, null=True)
    miniature = models.BooleanField(_(u"Display inside the description?"),
                                  default=settings.CHIMERE_MINIATURE_BY_DEFAULT)
    thumbnailfile = models.ImageField(_(u"Thumbnail"), upload_to='pictures',
                        blank=True, null=True,
                        height_field='thumbnailfile_height',
                        width_field='thumbnailfile_width')
    thumbnailfile_height = models.IntegerField(_(u"Thumbnail height"),
                                               blank=True, null=True)
    thumbnailfile_width = models.IntegerField(_(u"Thumbnail width"),
                                              blank=True, null=True)
    order = models.IntegerField(_(u"Order"), default=1)
    marker = models.ForeignKey(Marker, related_name='pictures')

    def __unicode__(self):
        return self.name or u""

    class Meta:
        verbose_name = _(u"Picture file")
        verbose_name_plural = _(u"Picture files")

def scale_image(max_x, pair):
    x, y = pair
    new_y = (float(max_x) / x) * y
    return (int(max_x), int(new_y))

IMAGE_EXIF_ORIENTATION_MAP = {
    1: 0,
    8: 2,
    3: 3,
    6: 4,
}

PYEXIV2_OLD_API = not hasattr(pyexiv2, 'ImageMetadata')

def picturefile_post_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    picturefile = kwargs['instance']

    if kwargs['created']:
        filename = picturefile.picture.path
        metadata, orientation = None, None
        if PYEXIV2_OLD_API:
            metadata = pyexiv2.Image(filename)
            metadata.readMetadata()
            orientation = metadata['Exif.Image.Orientation'] \
                   if 'Exif.Image.Orientation' in metadata.exifKeys() else None
        else:
            metadata = pyexiv2.ImageMetadata(filename)
            metadata.read()
            orientation = metadata['Exif.Image.Orientation'].value \
                      if 'Exif.Image.Orientation' in metadata else None
        if orientation and orientation in IMAGE_EXIF_ORIENTATION_MAP \
           and orientation > 1:
            metadata['Exif.Image.Orientation'] = 1
            if PYEXIV2_OLD_API:
                metadata.writeMetadata()
            else:
                metadata.write()
            im = Image.open(filename)
            im = im.transpose(IMAGE_EXIF_ORIENTATION_MAP[orientation])
            im.save(filename)

    if not picturefile.thumbnailfile:
        file = picturefile.picture
        # defining the filename and the thumbnail filename
        filehead, filetail = os.path.split(os.path.abspath(file.path))
        basename, format = os.path.splitext(filetail)
        basename = defaultfilters.slugify(basename)
        basename = re.sub(r'-','_', basename)
        miniature = basename + '_thumb' + format
        filename = file.path
        miniature_filename = os.path.join(filehead, miniature)
        try:
            image = Image.open(filename)
        except:
            image = None
        if image:
            image_x, image_y = image.size
            if settings.CHIMERE_THUMBS_SCALE_HEIGHT:
                image_y, image_x = scale_image(
                                          settings.CHIMERE_THUMBS_SCALE_HEIGHT,
                                          (image_y, image_x))
            elif settings.CHIMERE_THUMBS_SCALE_WIDTH:
                image_x, image_y = scale_image(
                                          settings.CHIMERE_THUMBS_SCALE_WIDTH,
                                          (image_x, image_y))
            image.thumbnail([image_x, image_y], Image.ANTIALIAS)

            temp_image = open(miniature_filename, 'w')
            try:
                image.save(temp_image, image.format, quality=90, optimize=1)
            except:
                image.save(temp_image, image.format, quality=90)

            short_name = miniature_filename[len(settings.MEDIA_ROOT):]
            picturefile.thumbnailfile = short_name
            picturefile.save()

    if not kwargs['created']:
        return
    pfs = PictureFile.objects.filter(marker=picturefile.marker).exclude(
                                     pk=picturefile.pk).order_by('order')
    for idx, pf in enumerate(pfs.all()):
        pf.order = idx + 1
        pf.save()
    picturefile.order = pfs.count() + 1
    picturefile.save()
post_save.connect(picturefile_post_save, sender=PictureFile)

class RouteFile(models.Model):
    name = models.CharField(_(u"Name"), max_length=150)
    raw_file = models.FileField(_(u"Raw file (gpx or kml)"),
                                upload_to='route_files')
    simplified_file = models.FileField(_(u"Simplified file"),
                        upload_to='route_files', blank=True, null=True)
    TYPE = (('K', _(u'KML')), ('G', _(u'GPX')))
    file_type = models.CharField(max_length=1, choices=TYPE)

    class Meta:
        ordering = ('name',)
        verbose_name = _(u"Route file")
        verbose_name_plural = _(u"Route files")

    def __unicode__(self):
        return self.name

    def process(self):
        if self.simplified_file:
            return
        input_name = settings.MEDIA_ROOT + self.raw_file.name
        output_name = settings.MEDIA_ROOT + self.raw_file.name[:-4] + \
                      "_simplified" + ".gpx"
        cli_args = [settings.GPSBABEL, '-i']
        if self.file_type == 'K':
            cli_args.append('kml')
        elif self.file_type == 'G':
            cli_args.append('gpx')
        cli_args += ['-f', input_name, '-x', settings.GPSBABEL_OPTIONS,
                     '-o', 'gpx', '-F', output_name]
        p = Popen(cli_args, stderr=PIPE)
        p.wait()
        if p.returncode:
            print p.stderr.read()
            #logger.error(p.stderr.read())
        else:
            self.simplified_file = File(open(output_name))
            self.save()
            os.remove(output_name)

    @property
    def route(self):
        if not self.simplified_file:
            return

        file_name = settings.MEDIA_ROOT + self.simplified_file.name
        tree = etree.parse(file_name)
        pts = []
        for pt in tree.getiterator():
            if not pt.tag.endswith('trkpt'):
                continue
            pts.append((pt.get("lon"), pt.get("lat")))
        geojson_tpl = u'{"type":"Feature", "geometry":{ "type": "LineString", '\
                       '"coordinates":[%s]}}'
        wkt_tpl = u'LINESTRING(%s)'
        return wkt_tpl % u','.join([u'%s %s' % (pt[0], pt[1]) \
                                           for pt in pts])

class Route(GeographicItem):
    '''Route on the map
    '''
    ref_item = models.ForeignKey("Route", blank=True, null=True,
              verbose_name=_(u"Reference route"), related_name='submited_route')
    route = RouteField(_(u"Route"),
                       srid=settings.CHIMERE_EPSG_DISPLAY_PROJECTION)
    associated_file = models.ForeignKey(RouteFile, blank=True, null=True,
                                        verbose_name=_(u"Associated file"))
    picture = models.ImageField(_(u"Image"), upload_to='upload', blank=True,
                          null=True, height_field='height', width_field='width')
    height = models.IntegerField(_(u"Height"), blank=True, null=True)
    width = models.IntegerField(_(u"Width"), blank=True, null=True)
    has_associated_marker = models.BooleanField(_(u"Has an associated marker"),
                                                default=True)
    objects = models.GeoManager()

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('status', 'name')
        verbose_name = _(u"Route")

    def __init__(self, *args, **kwargs):
        super(Route, self).__init__(*args, **kwargs)
        self.description = ''
        try:
            associated_marker = Marker.objects.get(route=self)
            self.description = associated_marker.description
        except:
            associated_marker = None
        # add read attributes for properties
        for pm in self.properties():
            attr_name = pm.getAttrName()
            if not hasattr(self, attr_name):
                val = ''
                if associated_marker:
                    property = associated_marker.getProperty(pm)
                    if property:
                        val = property.python_value
                setattr(self, attr_name, val)
            if not hasattr(self, attr_name + '_set'):
                setattr(self, attr_name + '_set',
                              property_setter(self.__class__, pm))

    @property
    def geometry(self):
        return self.route.wkt

    @property
    def geom_attr(self):
        return 'route'

    def get_init_multi(self):
        if not self.associated_marker.count():
            return []
        multis = [forms.model_to_dict(multi)
            for multi in self.associated_marker.all()[0].multimedia_files.all()]
        return multis

    def get_init_picture(self):
        if not self.associated_marker.count():
            return []
        picts = [forms.model_to_dict(pict)
                 for pict in self.associated_marker.all()[0].pictures.all()]
        return picts

    def getProperty(self, propertymodel, safe=None):
        """Get the property of an associated property model.
        If safe set to True, verify if the property is available
        """
        if safe and not propertymodel.available:
            return
        try:
            property = Property.objects.get(propertymodel=propertymodel,
                                            marker=self)
        except Property.DoesNotExist:
            return
        return property

    def getProperties(self):
        """Get all the property availables
        """
        properties = []
        for pm in PropertyModel.objects.filter(available=True):
            property = self.getProperty(pm)
            if property:
                properties.append(property)
        return properties

    def getGeoJSON(self, color="#000"):
        '''Return a GeoJSON string
        '''
        if '#' not in color:
            color = '#' + color
        attributes = {'id':self.id, 'name':json.dumps(self.name),
                      'color':color, 'geometry':self.route.geojson,}
        return u'{"type":"Feature", "geometry":%(geometry)s, '\
               u'"properties":{"pk": %(id)d, "name": %(name)s, '\
               u'"color":"%(color)s"}}' % attributes

    def getTinyUrl(self):
        parameters = 'current_feature=%d&checked_categories=%s' % (self.id,
                                                          self.categories[0].id)
        return TinyUrl.getUrnByParameters(parameters)

pre_save_route_values = {}
def route_pre_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    geometry_pre_save(Route, pre_save_route_values)(sender, **kwargs)
pre_save.connect(route_pre_save, sender=Route)

def route_post_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    geometry_post_save(pre_save_route_values)(sender, **kwargs)
    instance = kwargs['instance']

    # manage associated marker
    if instance.has_associated_marker:
        marker_fields = [f.attname for f in Marker._meta.fields]
        route_fields = [f.attname for f in Route._meta.fields]
        marker_dct = dict([(k, getattr(instance, k)) for k in marker_fields
                       if k in route_fields and k not in ('id', 'ref_item_id')])
        marker_dct['point'] = "SRID=%d;POINT(%f %f)" % (instance.route.srid,
                                  instance.route[0][0], instance.route[0][1])
        marker, created = Marker.objects.get_or_create(route=instance,
                                                       defaults=marker_dct)
        if not created:
            marker.status = instance.status
            marker.point = marker_dct['point']
        marker.save()
        properties = {}
        for pm in instance.properties():
            prop = instance.getProperty(pm)
            if prop:
                properties[pm.pk] = prop.python_value
        # fix mis-initialized markers
        if created:
            for cat in instance.categories.all():
                marker.categories.add(cat)
        marker.saveProperties(properties)

post_save.connect(route_post_save, sender=Route)

def sync_m2m_route(sender, **kwargs):
    if kwargs['action'] not in ('post_add', 'post_clear', 'post_remove'):
        return
    route = kwargs['instance']
    marker = route.associated_marker
    if not marker.count():
        return
    marker = marker.all()[0]
    marker.categories.clear()
    if kwargs['action'] == 'post_clear':
        return
    for cat in route.categories.all():
        marker.categories.add(cat)
m2m_changed.connect(sync_m2m_route, sender=Route.categories.through)

def getDateCondition():
    '''
    Return an SQL condition for apparition of dates
    '''
    if not settings.CHIMERE_DAYS_BEFORE_EVENT:
        return ""
    now = datetime.datetime.now().strftime('%Y-%m-%d')
    after = (datetime.datetime.now() + \
             datetime.timedelta(settings.CHIMERE_DAYS_BEFORE_EVENT)
                                                  ).strftime('%Y-%m-%d')
    date_condition = " and (%(alias)s.start_date is null or "
    date_condition += "(%(alias)s.start_date >= '" + now + "' and "
    date_condition += "%(alias)s.start_date <='" + after + "')"
    date_condition += " or (%(alias)s.start_date <='" + now + "' and "
    date_condition += "%(alias)s.end_date >='" + now + "')) "
    return date_condition

class SimplePoint:
    """
    Point in the map (not in the database)
    """
    def __init__(self, x, y):
        self.x, self.y = x, y

class SimpleArea:
    """
    Rectangular area of a map (not in the database)
    """
    def __init__(self, area):
        """
        Defining upper left corner ans lower right corner from a tuple
        """
        assert len(area) == 4
        x1, y1, x2, y2 = area
        self.upper_left_corner = SimplePoint(x1, y1)
        self.lower_right_corner = SimplePoint(x2, y2)

    def isIn(self, area):
        """
        Verify if the current area is in the designated area
        """
        if self.upper_left_corner.x >= area.upper_left_corner.x and \
           self.upper_left_corner.y <= area.upper_left_corner.x and \
           self.lower_right_corner.x <= area.lower_right_corner.x and \
           self.lower_right_corner.y >= area.lower_right_corner.y:
            return True
        return False

    def getCategories(self, status='A', filter_available=True, area_name=None):
        """
        Get categories for this area
        """
        wheres = []
        if area_name:
            subcategory_pks = []
            for cat, subcats in SubCategory.getAvailable(area_name=area_name):
                for subcat in subcats:
                    subcategory_pks.append(unicode(subcat.pk))
            if filter_available:
                wheres += ['subcat.available = TRUE',  'cat.available = TRUE']
            wheres += ['subcat.id in (%s)' % ",".join(subcategory_pks)]
        where = " where " + " and ".join(wheres) if wheres else ""

        equal_status = ''
        if len(status) == 1:
            equal_status = "='%s'" % status[0]
        elif status:
            equal_status = " in ('%s')" % "','".join(status)
        area = u"ST_GeometryFromText('POLYGON((%f %f,%f %f,%f %f,%f %f, %f %f"\
              u"))', %d)" % (self.upper_left_corner.x, self.upper_left_corner.y,
                          self.lower_right_corner.x, self.upper_left_corner.y,
                          self.lower_right_corner.x, self.lower_right_corner.y,
                          self.upper_left_corner.x, self.lower_right_corner.y,
                          self.upper_left_corner.x, self.upper_left_corner.y,
                          settings.CHIMERE_EPSG_DISPLAY_PROJECTION
                          )
        date_condition = getDateCondition()
        sql_main = '''select subcat.id as id, subcat.category_id as category_id,
        subcat.name as name, subcat.available as available,
        subcat.icon_id as icon_id, subcat.color_theme_id as color_theme_id,
        subcat.order as order, subcat.item_type as item_type
        from chimere_subcategory subcat
        inner join chimere_category cat on cat.id=subcat.category_id'''
        sql = sql_main + '''
        inner join chimere_marker mark on ST_Contains(%s, mark.point)''' % area
        if equal_status:
            sql += ' and mark.status' + equal_status
        sql += date_condition % {'alias':'mark'}
        sql += '''
        inner join chimere_marker_categories mc on mc.subcategory_id=subcat.id and
        mc.marker_id=mark.id'''
        sql += where
        subcats = set(SubCategory.objects.raw(sql))
        sql = sql_main + '''
        inner join chimere_route rt on (ST_Intersects(%s, rt.route) or
        ST_Contains(%s, rt.route))''' % (area, area)
        if equal_status:
            sql += ' and rt.status' + equal_status
        sql += date_condition % {'alias':'rt'}
        sql += '''
        inner join chimere_route_categories rc on rc.subcategory_id=subcat.id and
        rc.route_id=rt.id'''
        sql += where
        # subcats.union(set(SubCategory.objects.raw(sql)))
        # set union behave strangely. Doing it manualy...
        for c in set(SubCategory.objects.raw(sql)):
            subcats.add(c)
        return subcats

class Layer(models.Model):
    name = models.CharField(_(u"Name"), max_length=150)
    layer_code = models.TextField(_(u"Layer code"), max_length=300)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _("Layer")

class Area(models.Model, SimpleArea):
    """Rectangular area of the map
    """
    name = models.CharField(_(u"Name"), max_length=150)
    urn = models.SlugField(_(u"Area urn"), max_length=50, blank=True,
                           unique=True)
    welcome_message = models.TextField(_(u"Welcome message"), blank=True,
                                       null=True)
    order = models.IntegerField(_(u"Order"), unique=True)
    available = models.BooleanField(_(u"Available"))
    upper_left_corner = models.PointField(_(u"Upper left corner"),
            default='POINT(0 0)', srid=settings.CHIMERE_EPSG_DISPLAY_PROJECTION)
    lower_right_corner = models.PointField(_(u"Lower right corner"),
            default='POINT(0 0)', srid=settings.CHIMERE_EPSG_DISPLAY_PROJECTION)
    default = models.NullBooleanField(_(u"Default area"),
                    help_text=_(u"Only one area is set by default"))
    layers = SelectMultipleField(Layer, related_name='areas',
                                 through='AreaLayers', blank=True)
    default_subcategories = SelectMultipleField(SubCategory, blank=True,
                           verbose_name=_(u"Sub-categories checked by default"))
    dynamic_categories = models.NullBooleanField(
        _(u"Sub-categories dynamicaly displayed"),
        help_text=_(u"If checked, categories are only displayed in the menu if "
                    u"they are available on the current extent."))
    subcategories = SelectMultipleField(SubCategory, related_name='areas',
       blank=True, db_table='chimere_subcategory_areas',
       verbose_name=_(u"Restricted to theses sub-categories"),
       help_text=_(u"If no sub-category is set all sub-categories are "
                   u"available"))
    external_css = models.URLField(_(u"Link to an external CSS"), blank=True,
                                   null=True)
    restrict_to_extent = models.BooleanField(_(u"Restrict to the area extent"),
                                             default=False)
    objects = models.GeoManager()

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('order', 'name')
        verbose_name = _("Area")

    @classmethod
    def getAvailable(cls):
        '''Get available areas
        '''
        return cls.objects.filter(available=True)

    def getWkt(self):
        return "SRID=%d;POLYGON((%f %f,%f %f,%f %f,%f %f, %f %f))" % (
          settings.CHIMERE_EPSG_DISPLAY_PROJECTION,
          self.upper_left_corner.x, self.upper_left_corner.y,
          self.lower_right_corner.x, self.upper_left_corner.y,
          self.lower_right_corner.x, self.lower_right_corner.y,
          self.upper_left_corner.x, self.lower_right_corner.y,
          self.upper_left_corner.x, self.upper_left_corner.y,
          )

    def getIncludeMarker(self):
        """
        Get the sql statement for the test if the point is included in the area
        """
        return Q(point__contained=self.getWkt())

    def getIncludeRoute(self):
        """
        Get the sql statement for the test if the route is included in the area
        """
        return Q(route__contained=self.getWkt())

pre_save_area_values = {}
def area_pre_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    geometry_pre_save(Area, pre_save_area_values)(sender, **kwargs)
pre_save.connect(area_pre_save, sender=Area)

def area_post_save(sender, **kwargs):
    if not kwargs['instance']:
        return
    area = kwargs['instance']
    if area.default:
        defaults = Area.objects.filter(default=True).exclude(pk=area.pk)
        for default in defaults:
            default.default = False
            default.save()
    # manage permissions
    old_urn, old_name = area.urn, area.name
    if area.pk in pre_save_area_values:
        old_urn, old_name = pre_save_area_values[area.pk]
    perm, old_groups, old_users = None, [], []
    if area.urn != old_urn:
        oldmnemo = 'change_area_' + old_urn
        old_perm = Permission.objects.filter(codename=oldmnemo)
        if old_perm.count():
            perm = old_perm.all()[0]
            perm.codename = 'change_area_' + area.urn
            perm.save()
    if not area.urn:
        area.urn = defaultfilters.slugify(area.name)
        area.save()
    mnemo = 'change_area_' + area.urn
    perm = Permission.objects.filter(codename=mnemo)
    lbl = "Can change " + area.name
    if not perm.count():
        content_type, created = ContentType.objects.get_or_create(
                                        app_label="chimere", model="area")
        perm = Permission(name=lbl, content_type_id=content_type.id,
                          codename=mnemo)
        perm.save()
    else:
        perm = perm.all()[0]
        if old_name != area.name:
            perm.name = lbl
            perm.save()
    # manage moderation group
    groupname = area.name + " moderation"
    if old_name != area.name:
        old_groupname = old_name + " moderation"
        old_gp = Group.objects.filter(name=old_groupname)
        if old_gp.count():
            old_gp = old_gp.all()[0]
            old_gp.name = groupname
            old_gp.save()
    group = Group.objects.filter(name=groupname)
    if not group.count():
        group = Group.objects.create(name=groupname)
        group.permissions.add(perm)
        for app_label, model in (('chimere', 'marker'),
                                 ('chimere', 'route'),
                                 ('chimere', 'multimediafile'),
                                 ('chimere', 'picturefile'),
                                 ('chimere', 'routefile')):
            ct, created = ContentType.objects.get_or_create(app_label=app_label,
                                                            model=model)
            for p in Permission.objects.filter(content_type=ct).all():
                group.permissions.add(p)

post_save.connect(area_post_save, sender=Area)

def get_areas_for_user(user):
    """
    Getting subcats for a specific user
    """
    perms = user.get_all_permissions()
    areas = set()
    prefix = 'chimere.change_area_'
    for perm in perms:
        if perm.startswith(prefix):
            try:
                area = Area.objects.get(urn=perm[len(prefix):])
                areas.add(area)
            except ObjectDoesNotExist:
                pass
    return areas

def get_users_by_area(area):
    if not area:
        return []
    perm = 'change_area_'+area.urn
    return User.objects.filter(Q(groups__permissions__codename=perm)|
                                Q(user_permissions__codename=perm)).all()

class AreaLayers(models.Model):
    area = models.ForeignKey(Area)
    layer = models.ForeignKey(Layer)
    order = models.IntegerField(_(u"Order"))
    default = models.NullBooleanField(_(u"Default layer"))

    class Meta:
        ordering = ('order',)
        verbose_name = _("Layers")
        verbose_name_plural = _("Layers")

class PropertyModel(models.Model):
    '''Model for a property
    '''
    name = models.CharField(_(u"Name"), max_length=150)
    order = models.IntegerField(_(u"Order"))
    available = models.BooleanField(_(u"Available"))
    mandatory = models.BooleanField(_(u"Mandatory"))
    subcategories = SelectMultipleField(SubCategory, related_name='properties',
       blank=True, verbose_name=_(u"Restricted to theses sub-categories"),
       help_text=_(u"If no sub-category is set all the property applies to all "
                   u"sub-categories"))
    TYPE = (('T', _('Text')),
            ('L', _('Long text')),
            ('P', _('Password')),
            ('D', _("Date")))
    TYPE_WIDGET = {'T':forms.TextInput,
                   'L':TextareaWidget,
                   'P':forms.PasswordInput,
                   'D':DatePickerWidget}
    type = models.CharField(_(u"Type"), max_length=1, choices=TYPE)
    def __unicode__(self):
        return self.name
    class Meta:
        ordering = ('order',)
        verbose_name = _("Property model")

    def getAttrName(self):
        attr_name = defaultfilters.slugify(self.name)
        attr_name = re.sub(r'-','_', attr_name)
        return attr_name

    def getNamedId(self):
        '''Get the name used as named id (easily sortable)
        '''
        return 'property_%d_%d' % (self.order, self.id)

class Property(models.Model):
    '''Property for a POI
    '''
    marker = models.ForeignKey(Marker, verbose_name=_(u"Point of interest"))
    propertymodel = models.ForeignKey(PropertyModel,
                                      verbose_name=_(u"Property model"))
    value = models.TextField(_(u"Value"))
    def __unicode__(self):
        return "%s : %s" % (str(self.propertymodel), self.value)
    class Meta:
        verbose_name = _(u"Property")

    @property
    def python_value(self):
        if self.propertymodel.type == 'D':
            try:
                return datetime.date(*[int(val) for val in self.value.split('-')])
            except:
                return ""
        else:
            return self.value




#!/usr/bin/python
# -*- coding: utf-8 -*-

# This script is used to migrate properties data in description field
# It is only used when migration to version 2.0

import sys
sys.path.append('.')
sys.path.append('..')

from django.core.management import setup_environ
import settings

setup_environ(settings)

from django.db import connection, transaction

cursor = connection.cursor()

from chimere.models import PropertyModel, Property

sys.stdout.write("""
This script is used to migrate properties data in the new description
field.
It is only useful when migrating to version 2.0.

WARNING: once the data is migrated the property model and all the
associated data are removed.

WARNING: this should be used only on a new migrated system: all previous
data in the new description field will be overload by the data from the
property model.

""")
response = None
while response not in ('y', 'n'):
    sys.stdout.write("Do you want to continue (y/n)? ")
    response = raw_input()
if response == "n":
    sys.exit(0)
if not PropertyModel.objects.count():
    sys.stdout.write("There is no property model available.\n")
    sys.exit(0)

sys.stdout.write("\nAvailable property models:\n")
property_models = list(PropertyModel.objects.all())
for idx, property_model in enumerate(property_models):
    sys.stdout.write(" * %d - %s\n" % (idx+1, property_model.name))

response = 0
while not (response > 0 and response <= len(property_models)):
    sys.stdout.write("Choose the property model to migrate: ")
    response = raw_input()
    try:
        response = int(response)
    except ValueError:
        response = 0

property_model = property_models[response-1]

while response not in ('y', 'n'):
    sys.stdout.write("Are you sure you want to migrate %s (y/n)? " % \
                     property_model.name)
    response = raw_input()
if response == "n":
    sys.exit(0)

idx = 0
for idx, property in enumerate(Property.objects.filter(
                                        propertymodel=property_model)):
    property.marker.description = property.value
    property.marker.save()

sys.stdout.write("* %d marker(s) updated.\n" % (idx+1))
Property.objects.filter(propertymodel=property_model).delete()
sys.stdout.write("* %d propertie(s) deleted.\n" % (idx+1))
property_model.delete()
sys.stdout.write("* Property model deleted.\n")




#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
sys.path.append('.')

from django.core.management import setup_environ
import settings

setup_environ(settings)

from django.db import connection, transaction

cursor = connection.cursor()

from main.models import Area, Marker, Route, Icon, SubCategory
from django.contrib.gis.geos import LineString

# early versions before 0.1: urn field doesn't exist for area

import htmlentitydefs, re

def slugfy(text, separator):
    ret = u""
    text = text.strip()
    for c in text.lower():
        try:
           ret += htmlentitydefs.codepoint2name[ord(c)][0]
        except:
            ret += c
    ret = re.sub("\W", " ", ret)
    ret = re.sub(" +", separator, ret)
    return ret.strip()

QUERY_CHECK_FIELD = """SELECT a.attname AS field FROM pg_class c, pg_attribute a
    WHERE c.relname = '%s' AND a.attnum > 0 AND a.attrelid = c.oid
          AND a.attname='%s';"""
QUERY_CHECK_TABLE = """SELECT c.relname FROM pg_class c
WHERE c.relname = '%s';"""


query = QUERY_CHECK_FIELD % ('main_area', 'urn')
cursor.execute(query)
transaction.commit_unless_managed()

row = cursor.fetchone()
if not row:
    query_update = "ALTER TABLE main_area ADD COLUMN urn VARCHAR(50) \
UNIQUE"
    cursor.execute(query_update)
    transaction.commit_unless_managed()
    areas = Area.objects.all()
    print " * urn field created in table main_area"
    for area in areas:
        urn = slugfy(area.name, "-")
        area.urn = urn
        area.save()
        print " * area %s urn is now: %s" % (area.name, area.urn)

    query = "ALTER TABLE main_area ALTER COLUMN urn SET not null;"
    cursor.execute(query)
    transaction.commit_unless_managed()
    print " * urn field has now the constraint NOT NULL"

from django.contrib.auth.models import Permission, ContentType

# check if permission have been correctly set for each areas
areas = Area.objects.all()
for area in areas:
    content_type = ContentType.objects.get(app_label="main",
            model="area")
    mnemo = 'change_area_' + area.urn
    perm = Permission.objects.filter(codename=mnemo)
    if not perm:
        lbl = "Can change " + area.name
        perm = Permission(name=lbl, content_type_id=content_type.id,
                          codename=mnemo)
        perm.save()
        print ' * permission "' + lbl + '" has been created'
        print " WARNING: don't forget to update administrator's rights with \
this new permission"

# early versions before 0.1: subcategory_areas table doesn't exist
# version 1.0 subcategory_areas renamed to main_subcategory_areas

query = QUERY_CHECK_TABLE % 'main_subcategory_areas'
cursor.execute(query)
transaction.commit_unless_managed()

row = cursor.fetchone()
if not row:
    query = QUERY_CHECK_TABLE % 'subcategory_areas'
    cursor.execute(query)
    transaction.commit_unless_managed()

    row = cursor.fetchone()
    if row:
        query_rename = "ALTER TABLE subcategory_areas RENAME TO \
main_subcategory_areas;"
        cursor.execute(query_rename)
        transaction.commit_unless_managed()
        print " * subcategory_areas renamed to main_subcategory_areas"
    else:
        query_create = """
CREATE TABLE "main_subcategory_areas" (
    "id" serial NOT NULL PRIMARY KEY,
    "subcategory_id" integer NOT NULL REFERENCES "main_subcategory" ("id")
                                                  DEFERRABLE INITIALLY DEFERRED,
    "area_id" integer NOT NULL REFERENCES "main_area" ("id")
                                                  DEFERRABLE INITIALLY DEFERRED,
    UNIQUE ("subcategory_id", "area_id"));
"""
        cursor.execute(query_create)
        transaction.commit_unless_managed()
        print " * main_subcategory_areas created"

# early versions before 0.1: main_tinyurl table doesn't exist

query = QUERY_CHECK_TABLE % 'main_tinyurl'
cursor.execute(query)
transaction.commit_unless_managed()

row = cursor.fetchone()
if not row:
    query_create = """
CREATE TABLE "main_tinyurl" (
    "id" serial NOT NULL PRIMARY KEY,
    "parameters" varchar(500) NOT NULL);
"""
    cursor.execute(query_create)
    transaction.commit_unless_managed()
    print " * main_tinyurl created"


# early versions before 0.1: save area with wrong SRID
# only errors with default SRID is managed adapt the script for your SRID

from osgeo import osr

srs = osr.SpatialReference()
srs.ImportFromEPSG(4326) # WGS84
ll = srs.CloneGeogCS()
srs.ImportFromEPSG(settings.EPSG_PROJECTION)
proj = osr.CoordinateTransformation(srs, ll)

changed = False
areas = Area.objects.all()
for area in areas:
    # only one test: assume each point as been save with the same SRID...
    if area.upper_left_corner.srid == 4326 and area.upper_left_corner.x > 90 \
                                           or area.upper_left_corner.x < -90:
        changed = True
        pt = proj.TransformPoint(area.upper_left_corner.y,
                                 area.upper_left_corner.x)
        area.upper_left_corner.x = pt[0]
        area.upper_left_corner.y = pt[1]
        pt = proj.TransformPoint(area.lower_right_corner.y,
                                 area.lower_right_corner.x)
        area.lower_right_corner.x = pt[0]
        area.lower_right_corner.y = pt[1]
        area.save()
if changed:
    print " * projections of areas corrected"

# changement from version 1.0 to 1.1: add dated fields to markers and routes
if settings.DAYS_BEFORE_EVENT:
    for cls in (Marker, Route):
        table = cls._meta.db_table
        query = QUERY_CHECK_FIELD % (table, 'start_date')
        cursor.execute(query)
        transaction.commit_unless_managed()

        row = cursor.fetchone()
        if not row:
            query_update = "ALTER TABLE "+table+" ADD COLUMN start_date date"
            cursor.execute(query_update)
            transaction.commit_unless_managed()
            query_update = "ALTER TABLE "+table+" ADD COLUMN end_date date"
            cursor.execute(query_update)
            transaction.commit_unless_managed()
            print " * start_date and end_date added to table " + table + "."

# changement from version 1.0 to 1.1: add available_date field to marker
if 'chimere.rss' in settings.INSTALLED_APPS:
    for cls in (Marker,):
        table = cls._meta.db_table
        query = QUERY_CHECK_FIELD % (table, 'available_date')
        cursor.execute(query)
        transaction.commit_unless_managed()

        row = cursor.fetchone()
        if not row:
            query_update = "ALTER TABLE " + table + " ADD COLUMN \
available_date timestamp with time zone"
            cursor.execute(query_update)
            transaction.commit_unless_managed()
            print " * available_date added to table " + table + "."

# changement from version 1.0 to 1.1: version of django 1.2
# create specific height and width for image fields

for cls, attr in ((Icon, "image"), (Marker, "picture"),
                  (Route, "picture")):
    table = cls._meta.db_table
    query = QUERY_CHECK_FIELD % (table, 'width')
    cursor.execute(query)
    transaction.commit_unless_managed()

    row = cursor.fetchone()
    if not row:
        query_update = "ALTER TABLE "+table+" ADD COLUMN width integer"
        cursor.execute(query_update)
        transaction.commit_unless_managed()
        query_update = "ALTER TABLE "+table+" ADD COLUMN height integer"
        cursor.execute(query_update)
        transaction.commit_unless_managed()
        for obj in cls.objects.all():
            image = getattr(obj, attr)
            if not image:
                continue
            obj.width = image.width
            obj.height = image.height
            obj.save()
        print " * height and width of " + table + " corrected"

# changement from version 1.0 to 1.1: multiple selection of categories

for cls in (Marker, Route):
    table = cls._meta.db_table[len("main_"):]
    query = QUERY_CHECK_TABLE % ('main_' + table + '_categories')
    cursor.execute(query)
    transaction.commit_unless_managed()

    row = cursor.fetchone()
    if row:
        continue
    query_create = """
CREATE TABLE "main_%s_categories" (
  "id" serial NOT NULL PRIMARY KEY,
  "%s_id" integer NOT NULL REFERENCES "main_%s" ("id") DEFERRABLE INITIALLY DEFERRED,
  "subcategory_id" integer NOT NULL REFERENCES "main_subcategory" ("id") DEFERRABLE INITIALLY DEFERRED,
  UNIQUE ("%s_id", "subcategory_id"));
""" % (table, table, table, table)
    cursor.execute(query_create)
    transaction.commit_unless_managed()
    for obj in cls.objects.all():
        query = "select subcategory_id from main_%s where id=%d" % (table,
                                                                    obj.id)
        cursor.execute(query)
        transaction.commit_unless_managed()

        row = cursor.fetchone()
        if row:
            obj.categories.add(SubCategory.objects.get(id=row[0]))
            obj.save()
    query = "ALTER TABLE main_%s DROP COLUMN subcategory_id;" % table
    cursor.execute(query)
    transaction.commit_unless_managed()
    print " * main_%s_categories created" % table

# -> version 1.2: associate point to route (for the future)
query = QUERY_CHECK_FIELD % ('main_marker', 'route_id')
cursor.execute(query)
transaction.commit_unless_managed()

row = cursor.fetchone()
if not row:
    query_update = 'ALTER TABLE "main_marker" ADD COLUMN \
"route_id" integer REFERENCES "main_route" ("id") DEFERRABLE INITIALLY DEFERRED'
    cursor.execute(query_update)
    transaction.commit_unless_managed()
    print " * route_id added to table main_marker."

# -> version 1.3: file associated to routes
query = QUERY_CHECK_TABLE % 'main_routefile'
cursor.execute(query)
transaction.commit_unless_managed()

row = cursor.fetchone()
if not row:
    query_create = """
    CREATE TABLE "main_routefile" (
            "id" serial NOT NULL PRIMARY KEY,
            "name" varchar(150) NOT NULL,
            "raw_file" varchar(100) NOT NULL,
            "simplified_file" varchar(100),
            "file_type" varchar(1) NOT NULL
            )
    ;
    ALTER TABLE "main_route" ADD COLUMN
                 "associated_file_id" integer REFERENCES "main_routefile" ("id")
                                                  DEFERRABLE INITIALLY DEFERRED;
    """
    cursor.execute(query_create)
    transaction.commit_unless_managed()
    print " * main_routefile created"

# early versions before 0.1: save route with wrong SRID
# only errors with default SRID is managed adapt the script for your SRID

changed = False
routes = Route.objects.all()
for route in routes:
    # only one test: assume each point as been save with the same SRID...
    if route.route and route.route.srid == 4326 and \
       route.route[0][0] > 90 or route.route[0][0] < -90:
        changed = True
        new_route = []
        for pt in route.route:
            pt = proj.TransformPoint(pt[0], pt[1])
            new_route.append((pt[0], pt[1]))
        route.route = LineString(new_route)
        route.save()
if changed:
    print " * projections of routes corrected"



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

# ManytoMany between area and subcategories is now defined inside the class Area
class Migration(SchemaMigration):

    def forwards(self, orm):
        pass


    def backwards(self, orm):
        pass


    models = {
        'chimere.area': {
            'Meta': {'ordering': "('order', 'name')", 'object_name': 'Area'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'default': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'default_subcategories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layers': ('chimere.widgets.SelectMultipleField', [], {'symmetrical': 'False', 'related_name': "'areas'", 'blank': 'True', 'through': "orm['chimere.AreaLayers']", 'to': "orm['chimere.Layer']"}),
            'lower_right_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'subcategories': ('chimere.widgets.SelectMultipleField', [], {'symmetrical': 'False', 'related_name': "'areas'", 'blank': 'True', 'db_table': "'chimere_subcategory_areas'", 'to': "orm['chimere.SubCategory']"}),
            'upper_left_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'urn': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '50', 'blank': 'True'}),
            'welcome_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.arealayers': {
            'Meta': {'ordering': "('order',)", 'object_name': 'AreaLayers'},
            'area': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Area']"}),
            'default': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Layer']"}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.category': {
            'Meta': {'ordering': "['order']", 'object_name': 'Category'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.color': {
            'Meta': {'ordering': "['order']", 'object_name': 'Color'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '6'}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.colortheme': {
            'Meta': {'object_name': 'ColorTheme'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.icon': {
            'Meta': {'object_name': 'Icon'},
            'height': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'width': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.importer': {
            'Meta': {'object_name': 'Importer'},
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'filtr': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'importer_type': ('django.db.models.fields.CharField', [], {'max_length': '4'}),
            'source': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'srid': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'zipped': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'chimere.layer': {
            'Meta': {'object_name': 'Layer'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layer_code': ('django.db.models.fields.TextField', [], {'max_length': '300'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.marker': {
            'Meta': {'ordering': "('status', 'name')", 'object_name': 'Marker'},
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'import_key': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'import_source': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'import_version': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'point': ('chimere.widgets.PointField', [], {}),
            'ref_item': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'submited_marker'", 'null': 'True', 'to': "orm['chimere.Marker']"}),
            'route': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'associated_marker'", 'null': 'True', 'to': "orm['chimere.Route']"}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'submiter_comment': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'submiter_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'submiter_session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'})
        },
        'chimere.multimediafile': {
            'Meta': {'object_name': 'MultimediaFile'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'marker': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'multimedia_files'", 'to': "orm['chimere.Marker']"}),
            'miniature': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'multimedia_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.MultimediaType']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'chimere.multimediatype': {
            'Meta': {'object_name': 'MultimediaType'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iframe': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'media_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'mime_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.news': {
            'Meta': {'object_name': 'News'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.picturefile': {
            'Meta': {'object_name': 'PictureFile'},
            'height': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'marker': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'pictures'", 'to': "orm['chimere.Marker']"}),
            'miniature': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'width': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.property': {
            'Meta': {'object_name': 'Property'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'marker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Marker']"}),
            'propertymodel': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.PropertyModel']"}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        'chimere.propertymodel': {
            'Meta': {'ordering': "('order',)", 'object_name': 'PropertyModel'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '1'})
        },
        'chimere.route': {
            'Meta': {'ordering': "('status', 'name')", 'object_name': 'Route'},
            'associated_file': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.RouteFile']", 'null': 'True', 'blank': 'True'}),
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'import_key': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'import_source': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'import_version': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'ref_item': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'submited_route'", 'null': 'True', 'to': "orm['chimere.Route']"}),
            'route': ('chimere.widgets.RouteField', [], {}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'submiter_comment': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'submiter_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'submiter_session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.routefile': {
            'Meta': {'ordering': "('name',)", 'object_name': 'RouteFile'},
            'file_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'raw_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'simplified_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'chimere.subcategory': {
            'Meta': {'ordering': "['category', 'order']", 'object_name': 'SubCategory'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Category']"}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']", 'null': 'True', 'blank': 'True'}),
            'icon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Icon']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.tinyurl': {
            'Meta': {'object_name': 'TinyUrl'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameters': ('django.db.models.fields.CharField', [], {'max_length': '500'})
        }
    }

    complete_apps = ['chimere']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import date, timedelta

from logging import getLogger

from django import template
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db.models import Q, Count
from django.template.loader import render_to_string

from chimere.models import Marker, Area, News, SubCategory, MultimediaType
from chimere.widgets import get_map_layers

log = getLogger(__name__)

register = template.Library()

@register.inclusion_tag('chimere/blocks/areas.html', takes_context=True)
def display_areas(context):
    """
    Display available areas.
    """
    context_data = {"areas": Area.getAvailable(),
                    "base_url": reverse('chimere:index')
                    }
    if "area_name" in context:
        context_data['area_name'] = context["area_name"]
    return context_data

@register.inclusion_tag('chimere/blocks/submited.html', takes_context=True)
def submited(context):
    """
    Submited message.
    """
    return {"edit_url":reverse('chimere:edit'),
            "index_url":reverse('chimere:index')}

@register.inclusion_tag('chimere/blocks/welcome.html', takes_context=True)
def display_news(context, display=False):
    """
    Welcome message and active news.
    """
    area = None
    context_data = {'display':display}
    if "area_name" in context:
        try:
            area = Area.objects.get(urn=context["area_name"])
            context_data['area_name'] = context['area_name']
            context_data['welcome_message'] = area.welcome_message
        except ObjectDoesNotExist:
            pass
    # Retrieve news
    news = News.objects.filter(available=True)
    if area:
        news = news.filter(Q(areas__isnull=True)|Q(areas__in=[area.pk]))
    news = list(news.all())
    if settings.CHIMERE_DAYS_BEFORE_EVENT:
        # Retrieve active markers
        today = date.today()
        after = today + timedelta(settings.CHIMERE_DAYS_BEFORE_EVENT)
        q = Marker.objects.filter(status='A').filter(start_date__lte=after
                     ).filter(Q(end_date__gte=today)|
                     (Q(end_date__isnull=True) & Q(start_date__gte=today)))
        if area:
            q = q.filter(area.getIncludeMarker())
        news += list(q)
    news.sort(key=lambda x:x.date, reverse=True)
    context_data['news_lst'] = news
    return context_data

@register.inclusion_tag('chimere/blocks/head_jquery.html', takes_context=True)
def head_jquery(context):
    """
    Create context and display head elements (js, css, etc.) for jquery.
    """
    context_data = {
            "JQUERY_JS_URLS": settings.JQUERY_JS_URLS,
            "JQUERY_CSS_URLS": settings.JQUERY_CSS_URLS,
             }
    return context_data

@register.inclusion_tag('chimere/blocks/head_jme.html', takes_context=True)
def head_jme(context):
    """
    Create context and display head elements (js, css, etc.) for jme.
    """
    context_data = {
            "STATIC_URL": settings.STATIC_URL,
             }
    return context_data

@register.inclusion_tag('chimere/blocks/head_chimere.html', takes_context=True)
def head_chimere(context):
    """
    Create context and display head elements (js, css, etc.) for chimere.
    """
    area_name = context['area_name'] if 'area_name' in context else 'area_name'
    area = None
    if area_name:
        try:
            area = Area.objects.get(urn=area_name)
        except ObjectDoesNotExist:
            pass
    context_data = {
            "STATIC_URL": settings.STATIC_URL,
            "MEDIA_URL": settings.MEDIA_URL,
            "DYNAMIC_CATEGORIES": 'true' if area and area.dynamic_categories \
                                  else 'false',
            "EXTRA_URL": reverse("chimere:index"),
            "EPSG_DISPLAY_PROJECTION": settings.CHIMERE_EPSG_DISPLAY_PROJECTION,
            "EPSG_PROJECTION": settings.CHIMERE_EPSG_PROJECTION,
            "DEFAULT_CENTER": settings.CHIMERE_DEFAULT_CENTER,
            "DEFAULT_ZOOM": settings.CHIMERE_DEFAULT_ZOOM,
            "MAP_LAYER": settings.CHIMERE_DEFAULT_MAP_LAYER,
            "OSM_CSS_URLS": settings.OSM_CSS_URLS,
            "OSM_JS_URLS": settings.OSM_JS_URLS,
             }
    return context_data

@register.inclusion_tag('chimere/blocks/map.html', takes_context=True)
def map(context, map_id='map'):
    context_data =  {'map_id':map_id,
                     "STATIC_URL": settings.STATIC_URL}
    context_data['icon_offset_x'] = settings.CHIMERE_ICON_OFFSET_X
    context_data['icon_offset_y'] = settings.CHIMERE_ICON_OFFSET_Y
    context_data['icon_width'] = settings.CHIMERE_ICON_WIDTH
    context_data['icon_height'] = settings.CHIMERE_ICON_HEIGHT
    area_name = context['area_name'] if 'area_name' in context else 'area_name'
    map_layers, default_area = get_map_layers(area_name)
    context_data['map_layers'] = ", ".join(map_layers)
    if default_area:
        context_data['selected_map_layer'] = default_area
    context_data['p_checked_categories'] = None
    area = None
    if area_name:
        try:
            area = Area.objects.get(urn=area_name)
        except ObjectDoesNotExist:
            pass
    if not area:
        try:
            area = Area.objects.get(default=True)
        except ObjectDoesNotExist:
            pass
    if area:
        context_data['area_id'] = area_name
        if 'zoomout' in context and context['zoomout']:
            context_data['zoom'] = "[%s]" % ",".join((
                unicode(area.upper_left_corner.x),
                unicode(area.upper_left_corner.y),
                unicode(area.lower_right_corner.x),
                unicode(area.lower_right_corner.y)))
        if area.subcategories.filter(available=True).count() == 1:
            context_data['single_category'] = True
            context_data['p_checked_categories'] = "[%d]" % \
                                            area.subcategories.all()[0].pk
        elif area.default_subcategories.count():
            context_data['p_checked_categories'] = unicode([subcategory.pk
                    for subcategory in area.default_subcategories.all()])
        if area.restrict_to_extent:
            context_data['restricted_extent'] = """
var bounds = new OpenLayers.Bounds();
bounds.extend(new OpenLayers.LonLat(%f, %f));
bounds.extend(new OpenLayers.LonLat(%f, %f));
""" % (area.upper_left_corner.x, area.upper_left_corner.y,
       area.lower_right_corner.x, area.lower_right_corner.y)

    if SubCategory.objects.filter(available=True).count() <= 1:
        context_data['single_category'] = True
        if not context_data['p_checked_categories']:
            cat = ''
            if SubCategory.objects.filter(available=True).count():
                cat = unicode(SubCategory.objects.filter(available=True
                                                         ).all()[0].pk)
            context_data['p_checked_categories'] = "[%s]" % cat
    if not context_data['p_checked_categories']:
        context_data['p_checked_categories'] = "[]"
    context_data['dynamic_categories'] = 'true' \
                            if area and area.dynamic_categories else 'false'
    if 'request' not in context:
        return context_data
    request = context['request']
    # Default values
    context_data['p_current_route'] = context.get('current_route')
    if request.GET:
        for key in ('zoom', 'lon', 'lat', 'display_submited',
                    'current_feature'):
            if key in request.GET and request.GET[key]:
                context_data['p_'+key] = request.GET[key]
            else:
                context_data['p_'+key] = None
        if 'checked_categories' in request.GET \
           and request.GET['checked_categories']:
            cats = request.GET['checked_categories'].split('_')
            context_data['p_checked_categories'] = "[%s]" % ",".join(cats)
    return context_data

@register.inclusion_tag('chimere/blocks/multimedia_file.html',
                        takes_context=True)
def multimedia_render(context, multimedia_file):
    context['multimedia_item'] = multimedia_file
    return context

@register.inclusion_tag('chimere/blocks/alternate_multimedia.html')
def alternate_multimedia(formset_multi, formset_picture):
    return {'formsets':[formset_multi, formset_picture],
            "STATIC_URL": settings.STATIC_URL,
            'auto_type_id':MultimediaType.objects.get(name='auto').pk}

@register.simple_tag
def get_tinyfied_url(marker, area_name=''):
    if not marker or not hasattr(marker, 'get_absolute_url'):
        return ""
    url = marker.get_absolute_url(area_name)
    return url

@register.filter(name='ol_map')
def ol_map(item, arg='map_id'):
    geom, geom_type = None, None
    if hasattr(item, 'point'):
        geom, geom_type = item.point, 'point'
    elif hasattr(item, 'route'):
        geom, geom_type = item.route, 'route'
    rendered = render_to_string('chimere/blocks/ol_map.html', {'geom': geom,
                                                               'map_id':arg})
    return rendered



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        db.rename_table('main_news', 'chimere_news')
        db.rename_table('main_tinyurl', 'chimere_tinyurl')
        db.rename_table('main_colortheme', 'chimere_colortheme')
        db.rename_table('main_color', 'chimere_color')
        db.rename_table('main_category', 'chimere_category')
        db.rename_table('main_icon', 'chimere_icon')
        db.rename_table('main_subcategory', 'chimere_subcategory')
        db.rename_table('main_marker', 'chimere_marker')
        db.rename_table('main_routefile', 'chimere_routefile')
        db.rename_table('main_route', 'chimere_route')
        db.rename_table('main_propertymodel', 'chimere_propertymodel')
        db.rename_table('main_property', 'chimere_property')
        db.rename_table('main_area', 'chimere_area')
        db.rename_table('main_marker_categories', 'chimere_marker_categories')
        db.rename_table('main_route_categories', 'chimere_route_categories')
        db.rename_table('main_subcategory_areas', 'chimere_subcategory_areas')

    def backwards(self, orm):
        db.rename_table('chimere_news', 'main_news')
        db.rename_table('chimere_tinyurl', 'main_tinyurl')
        db.rename_table('chimere_colortheme', 'main_colortheme')
        db.rename_table('chimere_color', '_color')
        db.rename_table('chimere_category', 'main_category')
        db.rename_table('chimere_icon', 'main_icon')
        db.rename_table('chimere_subcategory', 'main_subcategory')
        db.rename_table('chimere_marker', 'main_marker')
        db.rename_table('chimere_routefile', 'main_routefile')
        db.rename_table('chimere_route', 'main_route')
        db.rename_table('chimere_propertymodel', 'main_propertymodel')
        db.rename_table('chimere_property', 'main_property')
        db.rename_table('chimere_area', 'main_area')
        db.rename_table('chimere_marker_categories', 'main_marker_categories')
        db.rename_table('chimere_route_categories', 'main_route_categories')
        db.rename_table('chimere_subcategory_areas', 'main_subcategory_areas')
    
    models = {
        'chimere.area': {
            'Meta': {'object_name': 'Area'},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lower_right_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'upper_left_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'urn': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'chimere.category': {
            'Meta': {'object_name': 'Category', 'db_table': "'main_category'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.color': {
            'Meta': {'object_name': 'Color', 'db_table': "'main_color'"},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '6'}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.colortheme': {
            'Meta': {'object_name': 'ColorTheme', 'db_table': "'main_colortheme'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.icon': {
            'Meta': {'object_name': 'Icon', 'db_table': "'main_icon'"},
            'height': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'width': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.marker': {
            'Meta': {'object_name': 'Marker', 'db_table': "'main_marker'"},
            'available_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'point': ('chimere.widgets.PointField', [], {}),
            'route': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Route']", 'null': 'True', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.news': {
            'Meta': {'object_name': 'News', 'db_table': "'main_news'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.property': {
            'Meta': {'object_name': 'Property', 'db_table': "'main_property'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'marker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Marker']"}),
            'propertymodel': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.PropertyModel']"}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        'chimere.propertymodel': {
            'Meta': {'object_name': 'PropertyModel', 'db_table': "'main_propertymodel'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '1'})
        },
        'chimere.route': {
            'Meta': {'object_name': 'Route', 'db_table': "'main_route'"},
            'associated_file': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.RouteFile']", 'null': 'True', 'blank': 'True'}),
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'route': ('chimere.widgets.RouteField', [], {}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.routefile': {
            'Meta': {'object_name': 'RouteFile', 'db_table': "'main_routefile'"},
            'file_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'raw_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'simplified_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'chimere.subcategory': {
            'Meta': {'object_name': 'SubCategory', 'db_table': "'main_subcategory'"},
            'areas': ('chimere.widgets.SelectMultipleField', [], {'symmetrical': 'False', 'related_name': "'areas'", 'blank': 'True', 'to': "orm['chimere.Area']"}),
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Category']"}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']", 'null': 'True', 'blank': 'True'}),
            'icon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Icon']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.tinyurl': {
            'Meta': {'object_name': 'TinyUrl', 'db_table': "'main_tinyurl'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameters': ('django.db.models.fields.CharField', [], {'max_length': '500'})
        }
    }
    
    complete_apps = ['chimere']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import template
from BeautifulSoup import BeautifulSoup, Comment
import re

register = template.Library()

def sanitize(value, allowed_tags):
    """Argument should be in form 'tag2:attr1:attr2 tag2:attr1 tag3', where tags
    are allowed HTML tags, and attrs are the allowed attributes for that tag.
    """
    js_regex = re.compile(r'[\s]*(&#x.{1,7})?'.join(list('javascript')))
    allowed_tags = [tag.split(':') for tag in allowed_tags.split()]
    allowed_tags = dict((tag[0], tag[1:]) for tag in allowed_tags)

    soup = BeautifulSoup(value)
    for comment in soup.findAll(text=lambda text: isinstance(text, Comment)):
        comment.extract()

    for tag in soup.findAll(True):
        if tag.name not in allowed_tags:
            tag.hidden = True
        else:
            tag.attrs = [(attr, js_regex.sub('', val)) for attr, val in tag.attrs
                         if attr in allowed_tags[tag.name]]
    return soup.renderContents().decode('utf8')

register.filter(sanitize)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import template
from django.utils.translation import ugettext as _
import re

register = template.Library()

@register.inclusion_tag('chimere/blocks/inline_formset.html')
def inline_formset(caption, formset):
    u"""
    Render a formset as an inline table.
    For i18n of the caption Be carreful to add manualy the caption label to the
    translated fields
    """
    return {'caption':caption, 'formset':formset}




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012 Étienne Loks <etienne.loks_AT_peacefrogsDOTnet>
# Copyright (C) 2010 Pierre Clarenc <pierre.crc_AT_gmailDOTcom>,
#                    Samuel Renard <renard.samuel_AT_gmailDOTcom>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.contrib.gis.geos import *
from django.contrib.gis.feeds import Feed
from django.contrib.syndication.views import FeedDoesNotExist
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from django.shortcuts import get_object_or_404

from chimere.models import Category, SubCategory, Marker, Area

class BaseFeed(Feed):
    """
    Base feed for Chimere objects
    """
    def item_link(self, item):
        ''' Return POI permalink '''
        coord = item.point
        cat = 0
        if item.categories.all() and item.categories.all()[0]:
            cat = item.categories.all()[0].pk
        return reverse('chimere:index') + '?zoom=16&lat=%f&lon=%f&'\
        'current_feature=%d&checked_categories=%d' % (coord.y, coord.x, item.id,
                                                      cat)

    def item_pubdate(self, item):
        """
        Date of the Marker when it has been available
        """
        return item.available_date

    def description(self, obj):
        return ""

    def item_geometry(self, obj):
        return obj.point

class LatestPOIsByCategory(BaseFeed):
    '''
    Last Points of interests by category in Feeds
    '''
    title_template = "chimere/feeds/rss_title.html"
    description_template = "chimere/feeds/rss_descr.html"

    def get_object(self, request, category_id, area_name=''):
        return get_object_or_404(Category, id=category_id)

    def title(self, obj):
        """
        Define the title of the feed
        """
        return u"%s - %s" % (settings.PROJECT_NAME, obj.name)

    def link(self, obj):
        """
        Define the link of the feed.
        """
        if not obj:
            raise FeedDoesNotExist
        return reverse('chimere:feeds-cat', args=['', obj.id])

    def items(self, obj):
        """
        Requests to marker where its category match the category is requested
        and its status is available
        This returns a list of the 15 last markers/POIs ordering by date
        """
        q = Marker.objects.filter(status__exact='A',
            categories__category__id__exact=obj.id,
            available_date__isnull=False).order_by('-available_date')[:15]
        return q

class LatestPOIsBySubCategory(BaseFeed):
    '''
    Last Points of interests by SubCategory in Feeds
    '''
    title_template = "chimere/feeds/rss_title.html"
    description_template = "chimere/feeds/rss_descr.html"

    def get_object(self, request, category_id, area_name=''):
        return get_object_or_404(SubCategory, id=category_id)

    def title(self, obj):
        return u"%s - %s - %s" % (settings.PROJECT_NAME, obj.category.name,
                                  obj.name)

    def link(self, obj):
        if not obj:
            raise FeedDoesNotExist
        return reverse('chimere:feeds-subcat', args=['', obj.id])

    def items(self, obj):
        q = Marker.objects.filter(categories__id__exact=obj.id,
            available_date__isnull=False, status__exact='A').order_by(
                                                        '-available_date')[:15]
        return q

class LatestPOIs(BaseFeed):
    '''
    Last Points of interests
    '''
    title_template = "chimere/feeds/rss_title.html"
    description_template = "chimere/feeds/rss_descr.html"

    def title(self):
        return settings.PROJECT_NAME + u" - " + _(u"Last points of interest")

    def link(self):
        return reverse('chimere:feeds-global')

    def description(self):
        return _("Latest points of interest from ") + settings.PROJECT_NAME

    def items(self):
        q = Marker.objects.filter(status__exact='A',
                  available_date__isnull=False).order_by('-available_date')[:15]
        return q

class LatestPOIsByZone(BaseFeed):
    '''
    Last Points of interests by zone by coordinates
    '''
    title_template = "chimere/feeds/rss_title.html"
    description_template = "chimere/feeds/rss_descr.html"
    upper_left_lat = 0
    upper_left_lon = 0
    lower_right_lat = 0
    lower_right_lon = 0

    def get_object(self, request, area, area_name=''):
        """
        Get the extra url. Parameters are the coordinates of the zone (the
        upper left and lower right points)
        """
        if not area:
            raise ObjectDoesNotExist
        # Then define the upper right and lower left points
        coordinates = str(area).split('_')
        upper_left_lat = float(coordinates[0])
        upper_left_lon = float(coordinates[1])
        lower_right_lat = float(coordinates[2])
        lower_right_lon = float(coordinates[3])
        upper_right_lat = upper_left_lat
        upper_right_lon = lower_right_lon
        lower_left_lat = lower_right_lat
        lower_left_lon = upper_left_lon
        # Define a Polygon with the 4 points of the zone.
        areaBox = Polygon(((upper_left_lon, upper_left_lat),
                            (upper_right_lon, upper_right_lat),
                            (lower_right_lon, lower_right_lat),
                            (lower_left_lon, lower_left_lat),
                            (upper_left_lon, upper_left_lat)),
                            srid=settings.CHIMERE_EPSG_DISPLAY_PROJECTION)
        return areaBox

    def title(self, obj):
        return settings.PROJECT_NAME + u" - " +\
               _(u"Last points of interest by area")

    def link(self, obj):
        """
        Define the link of the feed. It's the same url as we get in the method
        get_object
        """
        if not obj:
            raise FeedDoesNotExist
        area = str(self.upper_left_lat) + '_' + str(self.upper_left_lon) + \
               '_' + str(self.lower_right_lat) + '_' + str(self.lower_right_lon)
        return reverse('chimere:feeds-area', args=['', area])

    def items(self, obj):
        """
        Request to return Markers WHERE there points are containes in the zone
        which is requested.
        This returns a list of the 15 last markers/POIs ordering by date
        """
        q = Marker.objects.filter(point__contained=obj, status__exact='A',
                  available_date__isnull=False).order_by('-available_date')[:15]
        return q

class LatestPOIsByZoneID(BaseFeed):
    '''
    Last Points of interests by zone by id
    '''
    title_template = "chimere/feeds/rss_title.html"
    description_template = "chimere/feeds/rss_descr.html"

    def get_object(self, request, area_id, area_name=''):
        return get_object_or_404(Area, id=area_id)

    def title(self, obj):
        return settings.PROJECT_NAME + u" - " + \
              _(u"Last points of interest") + u" - " + obj.name

    def link(self, obj):
        if not obj:
            raise FeedDoesNotExist
        return reverse('chimere:feeds-areaid', args=['', obj.id])

    def items(self, obj):
        q = Marker.objects.filter(available_date__isnull=False, status='A')
        q = q.filter(obj.getIncludeMarker()).order_by('-available_date')[:15]
        return q
